# 数据结构
import time
from typing import Union
from pydantic import BaseModel, field_validator, model_validator

from and_tools import get_random_bin


class Cookies(BaseModel):
    skey: str = None
    client_key: str = None
    p_skey: dict = None


class UnTlvList(BaseModel):
    TGT_T10A: bytes = b''
    D2_T143: bytes = b''
    T100_qr_code_mark: bytes = b''  # watch
    T018: bytes = b''  # watch
    T019: bytes = b''  # watch
    T065: bytes = b''  # watch
    T108: bytes = b''
    userSt_Key: bytes = b''  # T10E
    wtSessionTicketKey: bytes = b''  # 134
    wtSessionTicket: bytes = b''  # 133

    userStSig: bytes = b''  # T114
    T16A: bytes = b''
    T106: bytes = b''
    T146: Union[str, dict] = None
    T192_captcha: str = None
    T104_captcha: bytes = b''
    T546_captcha: bytes = b''

    @model_validator(mode="before")
    @classmethod
    def validate_hex_fields(cls, values):
        for field in ['TGT_T10A',
                      'D2_T143',
                      'userSt_Key',
                      'userStSig',
                      'wtSessionTicket',
                      'wtSessionTicketKey']:
            if isinstance(values.get(field), str):
                values[field] = bytes.fromhex(values[field])
        return values


class Device(BaseModel):
    # 软件信息
    version: str = None
    package_name: str = None  # com.tencent.qqlite
    Sig: str = None
    build_time: int = None  # 软件构建时间 1654570540
    sdk_version: str = None  # #6.0.0.2366
    client_type: str = 'QQ'  # android
    app_id: int = None  # 似乎可以传空
    var: str = None
    # 设备信息
    name: str = 'android'
    internet: str = 'China Mobile GSM'
    internet_type: str = 'wifi'
    model: str = None
    brand: str = None
    Mac_bytes: bytes = None
    Bssid_bytes: bytes = None
    android_id: str = None  # Android 操作系统中设备的一个唯一ID。每个设备在首次启动时都会生成一个随机的64位数字作为其
    boot_id: str = None  # Linux系统中用来唯一标识系统自上次启动以来的运行时期的标识符
    Imei: str = None
    Mac: str = None
    Bssid: str = None


class NNXInfo(BaseModel):
    uin: str = None
    uin_name: str = None
    password: str = None
    seq: int = 5266
    share_key: bytes = None  # _D2Key
    key_rand: bytes = get_random_bin(16)
    key_tgtgt: bytes = None
    key_Pubkey: bytes = None  # 公钥
    guid: bytes = get_random_bin(16)
    login_time: int = int(time.time())
    UN_Tlv_list: UnTlvList = UnTlvList()
    device: Device = Device()
    cookies: Cookies = Cookies()
    Tips: str = None
    proxy_str: str = None
    proxy_proxies: dict = None
    emp_time: str = None

    @model_validator(mode="before")
    @classmethod
    def validate_hex_fields(cls, values):
        for field in ['share_key', 'guid']:
            if isinstance(values.get(field), str):
                values[field] = bytes.fromhex(values[field])
        return values



import random
import uuid


def calculate_luhn(partial_imei):
    """
    Calculate the Luhn checksum digit for a given IMEI prefix.

    Args:
        partial_imei (str): The first 14 digits of the IMEI.

    Returns:
        int: The Luhn checksum digit.
    """
    total = 0
    for i, digit in enumerate(partial_imei):
        n = int(digit)
        if i % 2 == 1:  # Double every second digit
            n *= 2
            if n > 9:  # Subtract 9 from numbers larger than 9
                n -= 9
        total += n
    return (10 - (total % 10)) % 10


def generate_china_imei():
    """
   生成一个适用于中国地区的随机有效IMEI号码。

    返回：
        str：一个带有中国特定TAC的有效IMEI号码。
    """
    # Select a TAC prefix commonly used in China
    china_tac_prefixes = [
        "860206",  # Example TAC for Chinese devices
        "867600",  # Example TAC for Huawei devices
        "868731",  # Example TAC for Xiaomi devices
        "861234"  # Example TAC for other Chinese brands
    ]
    tac = random.choice(china_tac_prefixes)

    # Generate the next 8 random digits (6 device + 2 variant)
    device_identifier = ''.join(str(random.randint(0, 9)) for _ in range(8))

    # Combine TAC and random device identifier to form the first 14 digits
    partial_imei = tac + device_identifier

    # Calculate the checksum digit
    checksum = calculate_luhn(partial_imei)

    # Append the checksum digit
    full_imei = partial_imei + str(checksum)
    return full_imei


def generate_china_mac():
    """
    生成一个带有中国基础 OUI 前缀的随机 MAC 地址。

    返回：
        str：一个有效的 MAC 地址。
    """
    # List of China-based OUI prefixes
    china_oui_prefixes = [
        "00:25:9E",  # Huawei
        "EC:26:CA",  # Huawei
        "74:23:44",  # Xiaomi
        "EC:3E:09",  # Xiaomi
        "00:17:EB",  # ZTE
        "9C:D2:1E",  # ZTE
        "C4:6E:1F",  # TP-Link
        "F4:F2:6D"  # TP-Link
    ]

    # Choose a random OUI prefix
    oui = random.choice(china_oui_prefixes)

    # Generate the remaining 3 bytes of the MAC address
    remaining_bytes = ":".join(f"{random.randint(0, 255):02X}" for _ in range(3))

    # Combine OUI and the remaining bytes
    mac_address = f"{oui}:{remaining_bytes}"
    return mac_address


def generate_china_bssid():
    """
    生成一个带有中国区域唯一标识符（OUI）前缀的随机BSSID。

    返回：
        str：一个有效的BSSID。
    """
    # List of China-based OUI prefixes for BSSID
    china_oui_prefixes = [
        "00:25:9E",  # Huawei
        "EC:26:CA",  # Huawei
        "74:23:44",  # Xiaomi
        "EC:3E:09",  # Xiaomi
        "00:17:EB",  # ZTE
        "9C:D2:1E",  # ZTE
        "C4:6E:1F",  # TP-Link
        "F4:F2:6D"  # TP-Link
    ]

    # Randomly select an OUI prefix
    oui = random.choice(china_oui_prefixes)

    # Generate the remaining 3 bytes of the BSSID
    remaining_bytes = ":".join(f"{random.randint(0, 255):02X}" for _ in range(3))

    # Combine the OUI and the remaining bytes
    bssid = f"{oui}:{remaining_bytes}"
    return bssid


def generate_random_device():
    """
    生成一个随机设备的品牌和型号。

    返回:
        dict: 一个包含随机品牌和型号的字典。
    """
    # Define brands and their corresponding models
    device_data = {
        "Samsung": ["Galaxy S22", "Galaxy Note 20", "Galaxy Z Fold4", "Galaxy A52", "Galaxy Tab S8"],
        "Xiaomi": ["Redmi Note 12", "Mi 11", "Poco X4", "Mi Pad 5", "Redmi K50"],
        "Huawei": ["P50 Pro", "Mate 50", "Nova 10", "Honor 70", "MatePad Pro"],
        "OnePlus": ["9 Pro", "Nord 2", "10T", "OnePlus 11", "OnePlus 8T"],
        "Sony": ["Xperia 1 III", "Xperia 5 IV", "Xperia 10 III", "Xperia Pro-I"],
        "Google": ["Pixel 6", "Pixel 7 Pro", "Pixel 5a", "Pixel Slate", "Pixelbook Go"],
    }

    # Randomly select a brand
    brand = random.choice(list(device_data.keys()))

    # Randomly select a model from the chosen brand
    model = random.choice(device_data[brand])

    # Return the random brand and model
    return {"brand": brand, "model": model}


def generate_boot_id():
    """
    生成一个随机的 boot_id 使用 UUID4。

    返回:
    str: 一个随机生成的 boot_id。
    """
    return str(uuid.uuid4())


def generate_android_id():
    """
    生成一个随机的Android ID。
    """
    return ''.join(random.choices('0123456789abcdef', k=16))



import json


class JceStructStatics:
    BYTE = 0
    SHORT = 1
    INT = 2
    LONG = 3
    FLOAT = 4
    DOUBLE = 5
    STRING1 = 6
    STRING4 = 7
    MAP = 8
    LIST = 9
    STRUCT_BEGIN = 10
    STRUCT_END = 11
    ZERO_TAG = 12
    SIMPLE_LIST = 13
    JCE_MAX_STRING_LENGTH = 104857600


class JceStruct(object):

    def __init__(self):
        self.data = {}

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        self.data[key] = value

    def read_from(self, stream):
        """读取"""
        self.data.clear()
        head_data, _ = stream.peak_head()
        while head_data.type != 11 and stream._bs.position < len(stream._bs.bytes):
            tag = head_data.tag
            value = stream.read_current(True)
            if isinstance(value, bytes) and len(value) > 0:
                try:
                    s = JceStruct()
                    from Jce.stream import JceInputStream
                    s.read_from(JceInputStream(value))
                    self.data[tag] = s
                except:
                    self.data[tag] = value
            else:
                self.data[tag] = value
            if stream._bs.position >= len(stream._bs.bytes):
                break
            head_data, _ = stream.peak_head()

    def to_json(self):
        return json.dumps(self, cls=JceStructEnconding, ensure_ascii=False)


class JceStructEnconding(json.JSONEncoder):
    def default(self, o):  # pylint: disable=E0202
        if isinstance(o, JceStruct):
            if len(o.data) == 1:
                return list(o.data.values())[0]
            else:
                return o.data
        if isinstance(o, (bytes, bytearray)):
            return str(o)



import datetime
import time

from and_tools import Pack, TEA, UnPack, get_md5
from android.utils import pack_
from android.utils.head import pack_head_login
from android.wlogin_sdk.tlv import TLV
from android.wlogin_sdk.tlv_res import UnTlv


def wtlogin_exchange_emp(info):
    """"
    更新令牌,todo 需要修改公钥和私钥
    :param info:
    """

    info.key_Pubkey = bytes.fromhex(
        '04 70 83 E0 93 38 B0 49 98 89 88 B7 8B 87 D8 B0 03 CE 45 B2 6D A6 92 21 84 67 A0 63 49 6F 78 B3 36 06 36 E2 19 8D 18 85 57 DA 0D 30 2D 2E 53 1E 2C C2 2C 21 4B 92 7F 8A 5B BC CC AD 33 19 AF F3 1A')
    _tlv = TLV(info)

    methods = [
        _tlv.T100(5, 16, 0, 34869472),
        _tlv.T10A(info.UN_Tlv_list.TGT_T10A),

        _tlv.T116(2),
        _tlv.T143(info.UN_Tlv_list.D2_T143),

        _tlv.T142(),
        _tlv.T154(),

        _tlv.T017(info.device.app_id, int(info.uin), info.login_time),
        _tlv.T141(),

        _tlv.T008(),

        _tlv.T147(),

        _tlv.T177(),
        _tlv.T187(),
        _tlv.T188(),
        _tlv.T202(),
        _tlv.T511()
    ]

    pack = Pack()
    pack.add_hex('00 0B')
    pack.add_int(len(methods), 2)  # 数量
    for method_result in methods:
        pack.add_bin(method_result)

    Buffer_tlv = pack.get_bytes()

    Buffer_tlv = TEA.encrypt(Buffer_tlv, bytes.fromhex('48 23 99 47 A6 E9 76 DF A5 43 26 F1 FB DE 51 18'))

    pack.empty()
    pack.add_hex('1F 41')
    pack.add_hex('08 10')
    pack.add_hex('00 01')
    pack.add_int(int(info.uin))
    pack.add_hex('03 07 00 00 00 00 02 00 00 00 00 00 00 00 00')
    pack.add_hex('02 01')

    pack.add_bin(info.key_rand)
    pack.add_hex('01 31')
    pack.add_hex('00 01')
    pack.add_body(info.key_Pubkey, 2)
    pack.add_bin(Buffer_tlv)
    Buffer = pack.get_bytes()

    pack.empty()
    pack.add_hex('02')
    pack.add_body(Buffer, 2, add_len=4)
    pack.add_hex('03')
    Buffer = pack.get_bytes()
    pack.empty()
    Buffer = pack_head_login(info, 'wtlogin.exchange_emp', Buffer)
    Buffer = pack_(info, data=Buffer, encryption=2, types=10, sso_seq=4)
    return Buffer


def wtlogin_exchange_emp_rsp(info, Buffer: bytes):
    # 其实和登录返回没区别
    Buffer = Buffer[15:-1]  # 头部 15字节&去掉尾部03
    _status = Buffer[0]

    Buffer = TEA.decrypt(Buffer[1:], bytes.fromhex('48 23 99 47 A6 E9 76 DF A5 43 26 F1 FB DE 51 18'))
    if _status == 0:
        Buffer = Buffer[5:]  # 00 09 00 00 02

        pack = UnPack(Buffer)
        _head = pack.get_bin(2).hex()
        _len = pack.get_short()
        Buffer = pack.get_bin(_len)

        if _head == '0119':
            # 判断tlv的头部
            Buffer = TEA.decrypt(Buffer, get_md5(info.share_key))

    else:
        Buffer = Buffer[3:]
    un_tlv = UnTlv(Buffer, info)
    un_tlv.unpack()
    if _status == 0:
        info.emp_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        return {'status': _status, 'cookie': info.cookies}

    return {'status': _status, 'message': '缓存更新异常'}


def wtlogin_trans(bArr):
    """k值 变体解法"""
    a_sm = b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff' \
           b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff>\xff\xff?\xff\xff456789:;<=\xff\xff\xff' \
           b'\xff\xff\xff\xff\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16' \
           b'\x17\x18\x19\xff\xff\xff\xff\xff\xff\x1a\x1b\x1c\x1d\x1e\x1f !"#$%&\'()*+,' \
           b'-./0123\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff' \
           b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff' \
           b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff' \
           b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff' \
           b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff' \
           b'\xff\xff\xff\xff\xff'
    if isinstance(bArr, str):
        bArr = bArr.encode()

    i = 32
    i4 = 0
    i3 = 0
    i2 = 0
    b = 0
    bArr2 = bytearray(24)
    while True:
        i6 = i - 1
        if i > 0:
            i7 = i4 + 1
            i5 = bArr[i4]

            if i5 != 0 or i5 == 95:
                if i5 == 32:
                    i5 = 42

                b2 = a_sm[i5]
                if b2 < 0:
                    b = b2
                    i = i6
                    i4 = i7
                else:
                    residue = i3 % 4
                    if residue == 0:
                        bArr2[i2] = b2 << 2
                        i5 = i2
                    elif residue == 1:
                        i5 = i2 + 1

                        bArr2[i2] = bArr2[i2] | (b2 >> 4)
                        bArr2[i5] = (b2 & 0x0F) << 4

                    elif residue == 2:
                        i5 = i2 + 1
                        bArr2[i2] = bArr2[i2] | (b2 >> 2)
                        bArr2[i5] = (b2 & 0x03) << 6
                    elif residue == 3:
                        i5 = i2 + 1
                        bArr2[i2] |= b2
                    else:
                        i5 = i2

                i3 += 1
                i4 = i7
                i = i6
                i2 = i5
                b = b2


        elif b == 95:
            residue = i3 / 4
            if residue == 1:
                break
            elif residue == 2:
                i2 = i2 + 1
        else:
            break

    return bArr2


def trans_emp_auth(info, **kwargs):
    verify = kwargs.get('verify', False)
    pack = Pack()
    pack.add_int(int(time.time()))

    if verify:
        pack.add_hex(
            '02 00 C9 00 14 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 03 00 00 00 32 00 00 00 02 00 00 00 00')
    else:
        pack.add_hex(
            '02 00 DE 00 13 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 03 00 00 00 32 00 00 00 00 00 00 00 00')

    pack.add_int(int(info.uin))
    pack.add_hex('00 00 00 00 00 10 00 00 00 00')
    pack.add_int(int(info.uin))
    pack.add_body(wtlogin_trans(kwargs['K']), 2)
    pack.add_body(info.UN_Tlv_list.TGT_T10A, 2)
    if verify:
        pack.add_hex('08 00 03 00 02 00 08 00 00 00 00 00 00 00 0B 00 15 00 04 00 00 00 00 00 68')
        pack.add_body(info.Guid, 2)
    else:
        pack.add_bin(info.Guid)
        pack.add_hex('01 00 01 08 00 04 00 03 00 05 00 20 00 36 00 01 00 09')
        pack.add_body('com.tencent.mobileqq', 2)  # 似乎会对其识别
        pack.add_hex('00 39 00 04 00 00 00 01')

    pack.add_hex('03')
    data = TEA.encrypt(pack.get_bytes(), info.UN_Tlv_list.userSt_Key)

    pack.empty()
    if verify:
        pack.add_hex('01 00 D8 00 00 00 10 00 00 00 72 00 60')
    else:
        pack.add_hex('01 00 F0 00 00 00 10 00 00 00 72 00 60')
    pack.add_bin(info.UN_Tlv_list.userStSig)
    pack.add_hex('00')
    pack.add_bin(data)

    data = TEA.encrypt(pack.get_bytes(), info.UN_Tlv_list.wtSessionTicketKey)

    pack.empty()
    pack.add_hex('1F 41')
    pack.add_hex('08 12')
    pack.add_hex('00 01')
    pack.add_int(int(info.uin))  # Uin_bytes
    pack.add_hex('03 45 00 00 00 00 02 00 00 00 00 00 00 00 00 00 30')
    pack.add_bin(info.UN_Tlv_list.wtSessionTicket)
    pack.add_bin(data)

    data = pack.get_bytes()

    pack.empty()  # 包裹
    pack.add_hex('02')
    pack.add_int(len(data) + 4, 2)  # 短整数
    pack.add_bin(data)
    pack.add_hex('03')

    data = pack.get_bytes()

    pack.empty()
    pack.add_hex(
        '00 00 00 27 00 00 00 15 77 74 6C 6F 67 69 6E 2E 74 72 61 6E 73 5F 65 6D 70 00 00 00 08 F7 C0 A1 E8 00 00 00 06 70 00')
    pack.add_int(len(data) + 4, 4)
    pack.add_bin(data)
    data = TEA.encrypt(pack.get_bytes(), '00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00')
    # 头部
    data = pack_(info, data=data, encryption=2, types=11, sso_seq=info.seq)
    return data


def trans_emp_auth_res(data, info, **kwargs):
    auth_info = {}
    verify = kwargs.get('verify', False)
    auth_info['verify'] = verify
    data = TEA.decrypt(data[16:-1], info.UN_Tlv_list.wtSessionTicketKey)
    data = TEA.decrypt(data[5:], info.UN_Tlv_list.userSt_Key)
    data = data[53:]
    pack = UnPack(data)
    status = pack.get_byte()

    if status != 0:
        _len = pack.get_short()
        message = pack.get_bin(_len).decode('utf-8')
        auth_info['message'] = message
    else:
        _time = pack.get_int(4)
        _len = pack.get_short()
        AuthName = pack.get_bin(_len).decode('utf-8')
        auth_info['AuthName'] = AuthName
        if verify:
            # 确认授权
            pack.get_short()
        data = pack.get_all()
        TLv = UnTlv(data, info)
        TLv.unpack()

        auth_info.update(TLv.get_auth_result())
    auth_info['status'] = status
    return auth_info



from .bytebuffer import ByteBuffer
from .struct import JceStruct
from .stream import JceInputStream


def jce_to_json(_data):
    """将 Jce 数据转换为 JSON 格式"""
    _stream = JceInputStream(_data)
    jce = JceStruct()
    jce.read_from(_stream)
    return jce.to_json()



# -*- coding: utf-8 -*-
from typing import Union, List, Optional, Any, MutableMapping

from .buffer import ByteBuffer
from .struct import IJceStruct

DEFAULT_ENCODING = "utf-8"


class JceWriter:
    """
    写入jce字节流
    """

    def __init__(self, data: Optional[Union[bytes, bytearray, ByteBuffer]] = None):
        if data is None:
            self.buffer = ByteBuffer()
        elif isinstance(data, (bytes, bytearray)):
            self.buffer = ByteBuffer(data)
        elif isinstance(data, ByteBuffer):
            self.buffer = data
        else:
            raise TypeError(f"can't init JceWriter with data type {data.__class__.__name__}")

    def write_head(self, type_: int, tag: int) -> None:
        """

        :param type_:
        :param tag:
        :return:
        """
        if tag < 15:
            data = bytes([tag << 4 | type_])  # go的byte就是uint8
            self.buffer.write_bytes(data)
        elif tag < 256:
            data = bytes([0xF0 | type_])  # 修改 0xF0 = 240
            self.buffer.write_bytes(data)
            self.buffer.write_bytes(bytes([tag]))

    def write_byte(self, b: bytes, tag: int) -> "JceWriter":
        """
        写入一个字节
        :param b:
        :param tag:
        :return:
        """
        if len(b) != 1:
            raise ValueError("write_byte only accept single byte")
        if b[0] == 0:
            self.write_head(12, tag)
        else:
            self.write_head(0, tag)
            self.buffer.write_bytes(b)
        return self

    def write_bool(self, b: bool, tag: int) -> None:
        if b:
            data: bytes = bytes([1])
        else:
            data: bytes = bytes([0])
        self.write_byte(data, tag)

    def write_int16(self, n: int, tag: int) -> None:
        if -128 <= n <= 127:
            self.write_byte(bytes([n]), tag)
            return
        self.write_head(1, tag)
        self.buffer.write_int2(n)

    def write_int32(self, n: int, tag: int) -> "JceWriter":
        if -32768 <= n <= 32767:
            self.write_int16(n, tag)
            return self

        self.write_head(2, tag)
        self.buffer.write_int4(n)
        return self

    def write_int64(self, n: int, tag: int) -> "JceWriter":
        if -2147483648 <= n <= 2147483647:
            return self.write_int32(n, tag)
        self.write_head(3, tag)
        self.buffer.write_int8(n)
        return self

    def write_float32(self, n: float, tag: int):
        self.write_head(4, tag)
        self.buffer.write_float(n)

    def write_float64(self, n: float, tag: int):  # 就是double
        self.write_head(5, tag)
        self.buffer.write_double(n)

    def write_string(self, s: str, tag: int) -> "JceWriter":
        """
        type 6 or 7 >255就得7了
        :param s:
        :param tag:
        :return:
        """
        by: bytes = s.encode(DEFAULT_ENCODING)
        if len(by) > 255:
            self.write_head(7, tag)
            self.buffer.write_bytes(len(by).to_bytes(4, "big"))  # 4个字节的长度
            self.buffer.write_bytes(by)
            return self
        self.write_head(6, tag)
        self.buffer.write_bytes(bytes([len(by)]))  # 1byte
        self.buffer.write_bytes(by)
        return self

    def write_bytes(self, data: Union[bytes, bytearray], tag: int):
        self.write_head(13, tag)
        self.write_head(0, 0)
        self.write_int32(len(data), 0)
        self.buffer.write_bytes(data)
        return self.buffer.bytes

    def write_int64_list(self, data: List[int], tag: int):
        """
        go: WriteInt64Slice
        :param data:
        :param tag:
        :return:
        """
        self.write_head(9, tag)
        if len(data) == 0:
            self.write_int32(0, 0)
            return
        self.write_int32(len(data), 0)
        for i in data:
            self.write_int64(i, 0)

    def write_list(self, data: List[Any], tag: int):
        if not isinstance(data, list):
            return
        self.write_head(9, tag)
        if len(data) == 0:
            self.write_int32(0, 0)
            return
        self.write_int32(len(data), 0)
        for i in data:
            self.write_object(i, 0)

    def write_jce_struct_list(self, data: List[IJceStruct], tag: int):
        self.write_head(9, tag)
        if len(data) == 0:
            self.write_int32(0, 0)
            return
        self.write_int32(len(data), 0)
        for i in data:
            self.write_jce_struct(i, 0)

    def write_map(self, m: dict, tag: int):
        if m is None:
            self.write_head(8, tag)
            self.write_int32(0, 0)
            return
        if not isinstance(m, MutableMapping):
            return
        self.write_head(8, tag)
        self.write_int32(len(m), 0)
        for k, v in m.items():
            self.write_object(k, 0)
            self.write_object(v, 1)
        return self.buffer.bytes

    def write_object(self, data: Any, tag: int):
        if isinstance(data, MutableMapping):
            self.write_map(data, tag)
            return
        if isinstance(data, list):
            self.write_list(data, tag)
            return
        if isinstance(data, (bytes, bytearray)):
            # 有时候字典单字节但是会传生成4字节,因此传bytearray来判断具体

            if len(data) == 1 and isinstance(data, bytes):
                self.write_byte(data, tag)
            else:
                self.write_bytes(data, tag)
            return
        if isinstance(data, bool):
            self.write_bool(data, tag)
        elif isinstance(data, int):
            self.write_int64(data, tag)
        elif isinstance(data, float):
            self.write_float64(data, tag)
        elif isinstance(data, str):
            self.write_string(data, tag)
        elif isinstance(data, IJceStruct):
            self.write_jce_struct(data, tag)

    def write_jce_struct_raw(self, data: IJceStruct):
        """
        只写内容不写头部
        todo 用pydantic给前面的都写加上jceid元数据 不然没法玩

        :param data:
        :return:
        """
        for field_name, val in data.__fields__.items():
            jce_id: int = val.field_info.extra["jce_id"]
            field_val = getattr(data, field_name)
            self.write_object(field_val, jce_id)

    def write_jce_struct(self, data: Union[bytes, bytearray], tag: int):
        self.write_head(10, tag)
        # 修改 原先的 write_jce_struct_raw 不知道是什么操作
        self.buffer.write_bytes(data)
        self.write_head(11, 0)
        return self.buffer.bytes

    def bytes(self) -> bytearray:
        """直接返回的数据对象"""
        return self.buffer.bytes



import struct


class ByteBuffer(object):
    _bytes = None
    _position = 0

    @property
    def bytes(self) -> bytes:
        return self._bytes

    @property
    def position(self):
        return self._position

    @position.setter
    def position(self, value):
        if not isinstance(value, int):
            raise TypeError("'position' 属性必须是整数")
        elif value < 0:
            raise ValueError("'position' 属性必须是正数")
        elif value > len(self._bytes):
            raise ValueError('')
        else:
            self._position = value

    def __init__(self, bs: bytes):
        if isinstance(bs, bytes):
            self._bytes = bs
        elif isinstance(bs, bytearray):
            self._bytes = bytes(bs)
        else:
            raise TypeError("'bs' 参数必须是字节或字节数组")

    def get(self):
        if self._position >= len(self._bytes):
            raise BufferError('到达字节末尾')
        b = self._bytes[self._position]
        self._position += 1
        return b

    def get_bytes(self, size):
        """获取字节"""
        if size < 0:
            raise ValueError("'size'属性必须是正数")
        if self._position > len(self._bytes):
            raise BufferError('到达字节末尾')
        if self.position + size > len(self._bytes):
            raise BufferError('到达字节末尾')
        b = self._bytes[self.position:self.position + size]
        # print(b.hex(),b)
        self.position = self.position + size
        return b

    def get_int2(self):
        b = self.get_bytes(2)
        return struct.unpack('>h', b)[0]

    def get_int4(self):
        b = self.get_bytes(4)
        return struct.unpack('>i', b)[0]

    def get_int8(self):
        b = self.get_bytes(8)
        return struct.unpack('>q', b)[0]

    def get_float(self):
        b = self.get_bytes(4)
        return struct.unpack('>f', b)[0]

    def get_double(self):
        b = self.get_bytes(8)
        return struct.unpack('>d', b)[0]

    def duplicate(self):
        bb = ByteBuffer(self._bytes)
        bb.position = self.position
        return bb

    def clear(self):
        self._position = 0



import datetime
import json
import random
import time

from box import Box
from loguru import logger

from and_tools import UnPack, TEA
from and_tools.device_infor import generate_china_imei, generate_china_mac, generate_china_bssid, \
    generate_random_device, generate_boot_id, generate_android_id
from android.Tcp import start_client, disconnect_client
from android.http.accounts import change_psw_getsms
from android.http.friends import Friends, bot_uin, delete_qq_friend
from android.http.level import GetLevelTask, qq_level_index, GetLevelStatus, RefreshTask
from android.http.music import Music
from android.http.qqsignin import WriteMindJar
from android.http.weishi import weishi_up_
from android.model import NNXInfo
from android.pack import *
from android.pack.SQQzoneSvc.delUgc import SQQzoneSvc_delUgc, SQQzoneSvc_delUgc_rsp
from android.pack.SQQzoneSvc.like import SQQzoneSvc_like, SQQzoneSvc_like_rsp
from android.pack.SQQzoneSvc.publishmood import SQQzoneSvc_publishmood, SQQzoneSvc_publishmood_rsp
from android.pack.StatSvc_register import StatSvc_register, StatSvc_register_rsp
from android.pack.SummaryCard import SummaryCard_ReqSummaryCard, SummaryCard_ReqSummaryCard_rsp
from android.pack.trpc_metaverse_mob_proxy_svr_MobProxy_SsoHandle import MobProxy_SsoHandle, MobProxy_SsoHandle_rsp
from android.struct.QQService import SvcReqGetDevLoginInfo, GetDevLoginInfo_profile, GetDevLoginInfo_res
from android.struct.SummaryCard import ReqSummaryCard
from android.struct.push import SvcReqRegister
from android.utils import qq_bkn
from android.wlogin_sdk.wtlogin import wtlogin_exchange_emp, wtlogin_exchange_emp_rsp, trans_emp_auth, \
    trans_emp_auth_res


class AndroidNNX:
    def __init__(self, proxy=None):
        if proxy is None:
            proxy = []

        self.info = NNXInfo()
        self.pack_list = {}  # 用于存储包的字典
        self._tcp = start_client(_func=self.un_data, proxy=proxy)

    def Tcp_send(self, data):
        self._tcp.sendall(data)
        start_time = time.time()  # 获取当前时间
        seq = self.info.seq
        while time.time() - start_time < 3:  # 检查是否已过去三秒
            data = self.pack_list.get(seq)
            if data is not None:
                self.pack_list.pop(seq)  # 删除已经取出的包
                break
            time.sleep(0.1)
        self.info.seq = seq + 1
        return data

    def scan_code_auth(self, **kwargs):
        """扫码授权"""

        def req_func(info):
            return trans_emp_auth(info, **kwargs)

        def rsp_func(buffer):
            return trans_emp_auth_res(buffer, self.info, **kwargs)

        return self.tcp_task(req_func, rsp_func)

    def tcp_task(self, req_func, rsp_func):
        buffer = req_func(self.info)
        if not self._tcp:
            return Box(status=-1, message='没有成功连接到服务器')

        buffer = self.Tcp_send(buffer)
        if buffer == b'':
            if self.info.Tips is not None:
                status = -99
                message = self.info.Tips
            else:
                status = -91
                message = '返回空包体'
            return Box(status=status, message=message)
        elif buffer is None:
            return Box(status=-1, message='未返回数据')
        response = rsp_func(buffer)
        return Box(status=0, message='请求成功', response=response)

    def un_data(self, data):
        """解包"""
        pack = UnPack(data)
        pack.get_int()
        pack_way = pack.get_byte()

        pack.get_byte()  # 00
        _len = pack.get_int()
        pack.get_bin(_len - 4)  # Uin bin
        _data = pack.get_all()
        if pack_way == 2:
            # 登录相关
            _data = TEA.decrypt(_data, '00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00')
        elif pack_way == 1:
            _data = TEA.decrypt(_data, self.info.share_key)
        else:
            _data = b''
            logger.info('未知的解密类型')

        if not _data:
            return
        else:
            pack = UnPack(_data)
            _len = pack.get_int()
            part1 = pack.get_bin(_len - 4)
            _len = pack.get_int()
            part2 = pack.get_bin(_len - 4)
            # part1
            pack = UnPack(part1)
            seq = pack.get_int()
            pack.get_int()
            _len = pack.get_int()
            Tips = pack.get_bin(_len - 4).decode('utf-8')
            _len = pack.get_int()
            Cmd = pack.get_bin(_len - 4).decode('utf-8')
            # print('包序号', seq, '包类型', Cmd, part2.hex())
            if Tips != '':
                seq = self.info.seq  # 推送到最后一个包
                self.info.Tips = Tips
                # log.warning(f'Tips:{Tips}')
            # part2
            # log.info('包序号', ssoseq, '包类型', Cmd, part2.hex())
            if 0 < seq < 1000000:
                # log.info('包序号', seq, '包类型', Cmd, part2.hex())
                self.pack_list.update({seq: part2})
            else:
                # log.info('推送包', seq, '包类型', Cmd, part2.hex())
                pass

    # 功能包
    def no_tail_login(self):
        """无尾登录包"""

        def req_func(info):
            return OidbSvc_0x88d_1(info, 790038285)

        return self.tcp_task(req_func, OidbSvc_0x88d_1_rep)

    def get_auth_list(self, start: int = 0, limit: int = 10):
        """
        获取授权列表
            参数:
                start = 0
                limit= 10
        """

        def req_func(info):
            return OidbSvc_0xc05(info, start, limit)

        return self.tcp_task(req_func, OidbSvc_0xc05_rep)

    def get_dev_login_info(self, iGetDevListType: int = 7):
        """
        获取设备登录信息
        参数:
            iGetDevListType: int, optional
                设备列表类型。如果未指定，将使用默认值 7。

        """

        def req_func(info):
            item = SvcReqGetDevLoginInfo(
                vecGuid=self.info.guid,
                iTimeStamp=1,
                strAppName='com.tencent.mobileqq',
                iRequireMax=20,
                iGetDevListType=iGetDevListType

            )
            return GetDevLoginInfo_profile(info, item)

        return self.tcp_task(req_func, GetDevLoginInfo_res)

    def unsubscribe(self, p_uin: int, cmd_type: int = 2):
        """
        取消订阅
            参数:
                p_uin: int 目标
                    2720152058 QQ团队
                    1770946116 安全中心
                    2290230341 QQ空间动态
                    2747277822 QQ手游
                    2010741172 QQ邮箱提醒


                cmd_type: int 默认2
        """

        def req_func(info):
            return OidbSvc_0xc96(info, p_uin, cmd_type)

        return self.tcp_task(req_func, OidbSvc_0xc96_rsp)

    def exchange_emp(self, forcibly: bool = False):
        """更新缓存"""
        if self.info.emp_time and not forcibly:
            last_emp_time = datetime.datetime.strptime(self.info.emp_time, "%Y-%m-%d %H:%M:%S")
            current_time = datetime.datetime.now()
            time_difference = current_time - last_emp_time
            if time_difference < datetime.timedelta(hours=12):
                # 12个小时内不更新
                return {'status': 0, 'message': '无需更新'}

        def req_func(info):
            return wtlogin_exchange_emp(info)

        def rsp_func(buffer):
            return wtlogin_exchange_emp_rsp(self.info, buffer)

        return self.tcp_task(req_func, rsp_func)

    def login_register(self, lBid: int = 7, iStatus: int = 11, iOSVersion: int = 25, bOnlinePush: bool = True):
        """
        登录注册
            参数
                lBid: int
                    默认:7
                    0 登出 7 登录
                iStatus: int
                    默认:11
                    11:在线
                    21:离线
                iOSVersion: int
                    默认:25
                bOnlinePush: bool  是否在线推送
                    默认:True

        """

        def req_func(info: NNXInfo):
            Buffer = SvcReqRegister(
                lUin=int(info.uin),
                lBid=lBid,
                iStatus=iStatus,
                iOSVersion=iOSVersion,
                cNetType=1,  # 网络类型
                vecGuid=info.guid,
                strOSVer='7.1.2',  # Build.VERSION.RELEASE todo 不应该固定
                bOnlinePush=bOnlinePush,
                iLargeSeq=41,

            ).to_bytes()

            return StatSvc_register(info, Buffer)

        return self.tcp_task(req_func, StatSvc_register_rsp)

    def get_summary_card(self, Uin: int = None, ComeFrom: int = 0):
        """"
        获取自身卡片
            参数:
            Uin: int, optional
                目标用户uin 不填获取自身
            ComeFrom: int, optional
                来源 0=自己 1=其他 31=搜索

        """

        def req_func(info):
            _ComeFrom = ComeFrom

            if Uin is None:
                _Uin = info.uin
            else:
                _Uin = Uin
                if ComeFrom == 0:
                    _ComeFrom = 1  # 其他

            Buffer = ReqSummaryCard(
                Uin=_Uin,
                ComeFrom=_ComeFrom,  # 自身是0,别人是其他 暂时不管细节
                IsFriend=True,
                GetControl=69181,
                AddFriendSource=10004,
                # stLocaleInfo=User_Locale_Info,
                SecureSig=bytes.fromhex('00'),
                ReqMedalWallInfo=True,  # 勋章墙,
                ReqNearbyGodInfo=True,  # 附近的信息
                ReqExtendCard=True,  # 请求扩展卡
                RichCardNameVer=True  # 富卡名称验证

            ).to_bytes()
            return SummaryCard_ReqSummaryCard(info, Buffer)

        return self.tcp_task(req_func, SummaryCard_ReqSummaryCard_rsp)

    def q_zone_like(self, target_Uin: int, title: str, feedskey: str):
        """
        空间点赞
        """

        def req_func(info):
            return SQQzoneSvc_like(info, target_Uin, title, feedskey)

        return self.tcp_task(req_func, SQQzoneSvc_like_rsp)

    def Qzone_publishmood(self, content: str):
        """
        空间发布信息
            :content 发送的内容
        """

        def req_func(info):
            return SQQzoneSvc_publishmood(info, content)

        return self.tcp_task(req_func, SQQzoneSvc_publishmood_rsp)

    def qq_show_replace(self, show='女性带眼镜'):
        """
        更换QQ秀
            dict_keys(['女性带眼镜', '女性_紫星连衣裙', '女性_带口罩', '女性_兔兔发箍', '女性_元气针织帽', '女性_白色百褶裙'])


        """

        def req_func(info):
            return MobProxy_SsoHandle(info, show)

        def rsp_func(buffer):
            return MobProxy_SsoHandle_rsp(buffer, show)

        return self.tcp_task(req_func, rsp_func)

    def lv_continuous_login(self):

        self.get_summary_card()
        self.get_lv_task()

    def Qzone_delUgc(self, srcId: str):
        """
        空间删除信息
            :srcId 发送说说时会返回:tid
        """

        def req_func(info):
            return SQQzoneSvc_delUgc(info, srcId)

        return self.tcp_task(req_func, SQQzoneSvc_delUgc_rsp)

    def lv_Qzone(self, content=None):
        """空间任务"""
        content_list = [
            ' 小荷才露尖尖角，早有蜻蜓立上头',
            ' 有三秋桂子，十里荷花',
            ' 藕花珠缀，犹似汗凝妆',
            ' 一缕清香，一缕淡香',
            ' 接天莲叶无穷碧，映日荷花别样红',
            '能不能停下来，看看那个满眼泪花奔向你的我。',
            '承诺常常很像蝴蝶，美丽的盘旋后就不见了。'
        ]
        random_content = random.choice(content_list)
        if not content:
            content = random_content

        logger.info(f'空间任务内容：{content}')
        try:
            rsp = self.Qzone_publishmood(content).get('response', {})
            tid = rsp.get('Busi', {}).get('tid', '')
            self.get_qq_level()
            self.Qzone_delUgc(tid)
            task = self.refresh_task('2').get('response', {}).get('task', {})
            if not task:
                return Box(status=False, message='刷新任务异常')
            button_text = task.get('button_text')
            if button_text == '去领取':
                finished_accelerate_days = task.get('accelerate_days', 0)
            else:
                finished_accelerate_days = 0
            return Box(status=True, message=f'新增活跃天数{finished_accelerate_days}天',
                       finishedAccelerateDays=finished_accelerate_days)
        except Exception as e:
            return Box(status=False, message=f'空间任务异常{e}')

    def lv_clock_in(self):
        """打卡任务"""
        self.sign_in()
        self.refresh_task('3')

    def lv_weishi(self):
        self.weishi_up()
        self.get_lv_task()

    def lv_Friend(self):
        """加好友任务"""
        selected = random.sample(bot_uin, 3)
        try:
            Friend = Friends(self.info, self.get_bkn(), self.info.uin, self.get_cookie('qun.qq.com'))
            success = 0
            for f_uin in selected:
                rsp = Friend.del_friend(f_uin)
                if not rsp.status:
                    continue
                rsp = Friend.add_friend(f_uin)
                if not rsp.status:
                    continue
                success += 1
            task = self.refresh_task('1').get('response', {}).get('task', {})
            finished_accelerate_days = task.get('finished_accelerate_days', 0)
            return Box(status=True, message=f'新增活跃天数{finished_accelerate_days}天',
                       finishedAccelerateDays=finished_accelerate_days)

        except Exception as e:
            return Box(status=False, message=f'好友活跃任务异常{e}')

        finally:
            for f_uin in selected:
                delete_qq_friend(self.info, self.info.uin, f_uin, self.get_bkn(), self.get_cookie('qzone.qq.com'))

    def lv_music(self):
        """音乐任务"""
        try:

            self.Music_task()
            finished_accelerate_days = 0
            for i in range(15):
                logger.info('正在等待音乐任务完成，剩余次数{}'.format(20 - i))
                time.sleep(1)
                task = self.refresh_task('19').get('response', {}).get('task', {})
                button_text = task.get('button_text')
                if button_text == '已完成':
                    finished_accelerate_days = task.get('accelerate_days', 0)
                    break
            logger.success(f'音乐任务完成,新增活跃{finished_accelerate_days}天')
            return Box(status=True, message=f'新增活跃天数{finished_accelerate_days}天',
                       finishedAccelerateDays=finished_accelerate_days)

        except Exception as e:
            return Box(status=False, message=f'音乐任务异常{e}')

    def lv_cm_show(self):
        """"厘米秀装扮"""
        try:
            shows = ['女性带眼镜', '女性_紫星连衣裙', '女性_带口罩', '女性_兔兔发箍', '女性_元气针织帽',
                     '女性_白色百褶裙']
            for i in range(2):
                show = random.choice(shows)
                shows.remove(show)
                self.qq_show_replace(show)
            task = self.refresh_task('27').get('response', {}).get('task', {})
            if not task:
                return Box(status=False, message='刷新任务异常')
            button_text = task.get('button_text')
            if button_text == '已完成':
                finished_accelerate_days = task.get('accelerate_days', 0)
            else:
                finished_accelerate_days = 0
            return Box(status=True, message=f'新增活跃天数{finished_accelerate_days}天',
                       finishedAccelerateDays=finished_accelerate_days)
        except Exception as e:
            return Box(status=False, message=f'厘米秀任务异常{e}')

    def lv_extra_task_up(self, lv_music=True, zone_content=None):
        """升级所有额外任务"""
        rsp = self.exchange_emp()
        if rsp['status'] != 0:
            return Box(status=False, message='emp' + rsp['message'])
        rsp = self.no_tail_login()
        status = rsp['status']
        if status != 0:
            return Box(status=False, message=rsp['message'], status_int=status)

        task_functions = {
            '连续登录QQ': self.lv_continuous_login,
            '发布一条空间说说': lambda: self.lv_Qzone(content=zone_content),
            '去日签卡打一次卡': self.lv_clock_in,
            '去QQ音乐听歌30分钟+0.5天': self.lv_music,
            '加一位好友': self.lv_Friend,
            '去超级QQ秀更新装扮并保存': self.lv_cm_show,
            '去微视APP看视频': self.lv_weishi,
        }

        task_status = self.get_lv_task_status(True)
        execute_task = False
        for task_name, function in task_functions.items():

            if task_name == '去QQ音乐听歌30分钟+0.5天' and not lv_music:
                logger.info(f'{self.info.uin[:2]}跳过音乐')
                continue  # 如果任务是'去QQ音乐听歌30分钟+0.5天'并且设置为不可选执行，则跳过执行
            is_done = task_status.get(task_name, {}).get('is_done', None)
            if not is_done:
                logger.info(f'{self.info.uin[:2]}执行{task_name}')
                function()
                execute_task = True

        level = self.get_summary_card().get('response', {}).get('5', 0)
        if execute_task:
            task_status = self.get_lv_task_status(True)

        done_count = task_status.get('done_count', None)
        logger.info(f'{self.info.uin[:2]}已完成任务数量{done_count}')
        return Box(status=True, level=level, done_count=done_count, task_status=task_status)

    # http
    def get_bkn(self):
        """
        获取bkn
        :return:
        """
        return qq_bkn(self.info.cookies.skey)

    def get_lv_task(self):
        """获取等级任务"""
        return GetLevelTask(self.info, self.get_cookie("ti.qq.com"))

    def get_qq_level(self):
        """
        获取qq等级
        :return .get('levelInfo')
        """
        return qq_level_index(self.info, self.get_cookie("ti.qq.com"))

    def get_lv_task_status(self, query: bool = False):
        """获取等级任务状态"""
        return GetLevelStatus(self.info, self.get_cookie("ti.qq.com"), query)

    def refresh_task(self, task_id: str):
        """刷新任务状态"""
        return RefreshTask(self.info, self.get_cookie("ti.qq.com"), task_id)

    def sign_in(self):
        """签到"""
        return WriteMindJar(self.info, self.get_cookie("ti.qq.com"), self.info.cookies.skey)

    def weishi_up(self):
        """微视任务"""
        return weishi_up_(self.info)

    def change_psw_getsms(self, mobile):
        """修改密码，获取验证码
        :param mobile: 手机号
        """
        return change_psw_getsms(self.info, self.get_cookie("accounts.qq.com"), mobile)

    def Music_task(self):
        """音乐"""
        return Music(self.info).task()

    def get_clientkey(self, domain: str = 'https://qzone.qq.com'):
        """"
        获取客户端key
        :param domain: 域名
       """
        return f'https://ssl.ptlogin2.qq.com/jump?clientuin={self.info.uin}&clientkey={self.info.cookies.client_key}&keyindex=19&pt_mq=0&u1={domain}'

    def get_cookie(self, domain: str):
        """
        :param domain: 域名
        :return:
        """
        p_skey = self.info.cookies.p_skey.get(domain, None)
        if p_skey is None:
            return None
        return f"uin=o{self.info.uin}; skey={self.info.cookies.skey}; p_uin=o{self.info.uin}; p_skey={p_skey};"

    # Tools
    def set_token_a(self, data):
        """
        设置TokenA
        """
        json_data = json.loads(data)
        if json_data['mark'] == 1013:

            self.info = NNXInfo.model_validate(json_data)
            default_values = {
                'self.info.device.Imei': generate_china_imei,
                'self.info.device.boot_id': generate_boot_id,
                'self.info.device.Bssid': generate_china_bssid,
                'self.info.device.Mac': generate_china_mac,
                'self.info.device.android_id': generate_android_id,
                'self.info.device.model': lambda: device_info.get('model'),
                'self.info.device.brand': lambda: device_info.get('brand'),
                'self.info.UN_Tlv_list.wtSessionTicket': lambda: bytes.fromhex(
                    '8EED6A0746FD906D06512F5F074BAD0F2D1729FA106EE98D40C9A5221F367579703360E29F4B7D4AE7FC25AE2D8DF241'
                ),
                'self.info.UN_Tlv_list.wtSessionTicketKey': lambda: bytes.fromhex(
                    '04BEBF0116413CF54C3D21919F0164D8'
                ),
            }

            device_info = generate_random_device()  # 生成随机设备
            for field, default in default_values.items():
                if not eval(field):
                    logger.warning(f'丢失{field}')
                    exec(f"{field} = default()")

                # 暂时固定
            self.info.device.sdk_version = '6.0.0.2497'
            self.info.device.package_name = 'com.tencent.mobileqq'
            self.info.device.build_time = 1645432578
            self.info.device.Sig = 'A6 B7 45 BF 24 A2 C2 77 52 77 16 F6 F3 6E B6 8D'
            self.info.device.version = '8.8.85'
            self.info.device.sdk_version = '6.0.0.2497'
            self.info.device.var = '||A8.9.71.9fd08ae5'
            return
        logger.error(f'不支持的mark{json_data["mark"]}')

    def get_token_a(self):
        tokenA = {
            'uin': self.info.uin,
            'password': self.info.password,
            'guid': self.info.guid.hex(),
            'share_key': self.info.share_key.hex(),
            'device': {
                'app_id': self.info.device.app_id,
                'Imei': self.info.device.Imei,
                'boot_id': self.info.device.boot_id,
                'Bssid': self.info.device.Bssid,
                'Mac': self.info.device.Mac,
                'android_id': self.info.device.android_id,
                'model': self.info.device.model,
                'brand': self.info.device.brand,

            },
            'UN_Tlv_list': {
                'TGT_T10A': self.info.UN_Tlv_list.TGT_T10A.hex(),
                'D2_T143': self.info.UN_Tlv_list.D2_T143.hex(),
                'userSt_Key': self.info.UN_Tlv_list.userSt_Key.hex(),
                'userStSig': self.info.UN_Tlv_list.userStSig.hex(),
                'wtSessionTicket': self.info.UN_Tlv_list.wtSessionTicket.hex(),
                'wtSessionTicketKey': self.info.UN_Tlv_list.wtSessionTicketKey.hex(),
            },
            'cookies': self.info.cookies.__dict__,
            'emp_time': self.info.emp_time,
            'mark': 1013,  # 解析标识
        }
        return json.dumps(tokenA)

    def close(self):
        clients_info = {}
        if self._tcp:
            clients_info = disconnect_client(self._tcp)
        return Box({'status': 0, 'message': 'AndroidQQ已释放', 'clients_info': clients_info})

    @property
    def tcp(self):
        return self._tcp



# -*- coding: utf-8 -*-
"""
https://tarscloud.github.io/TarsDocs/base/tars-protocol.html
"""
from typing import Tuple, Union, Callable, Any, List, Type

from .head import HeadData
from .buffer import ByteBuffer
from .struct import IJceStruct

DEFAULT_ENCODING = "utf-8"


class JceReader:
    """
    读取jce字节流
    """

    def __init__(self, data: Union[bytes, bytearray, ByteBuffer]):
        if isinstance(data, (bytes, bytearray)):
            self.buffer = ByteBuffer(data)
        elif isinstance(data, ByteBuffer):
            self.buffer = data
        else:
            raise TypeError(f"can't init JceReader with data type {data.__class__.__name__}")

    def read_head(self) -> Tuple[HeadData, int]:
        """
        不仅康了,而且指针动了
        :return:
        """
        head_data = HeadData()
        b: int = self.buffer.read()
        head_data.type = b & 0x0F  # 低4位位类型
        head_data.tag = (b & 0xF0) >> 4  # 高4位为tag,
        if head_data.tag == 15:  # 如果tag为15 则下一个字段为tag
            b: int = self.buffer.read()
            head_data.tag = b & 0xFF  # TODO 这个似乎可以去掉 不过昵昵不改我也不改 防止溢出的
            return head_data, 2
        else:
            return head_data, 1

    def peak_head(self) -> Tuple[HeadData, int]:
        """
        就康一眼
        :return:
        """
        return self.__class__(self.buffer.copy()).read_head()

    def skip(self, size: int) -> None:
        """
        跳过size个字节
        :param size:
        :return:
        """
        self.buffer.read_bytes(size)

    def _skip_field(self, type_: int):
        """
        跳过一个字段 仅仅是跳过内容 这个头部还得你自己跳的
        see https://blog.csdn.net/jiange_zh/article/details/86562232
        :param type_:
        :return:
        """
        if type_ == 0:
            self.skip(1)
        elif type_ == 1:
            self.skip(2)
        elif type_ in (2, 4):
            self.skip(4)
        elif type_ in (3, 5):
            self.skip(8)
        elif type_ == 6:
            len_ = self.buffer.read()
            self.skip(len_)
        elif type_ == 7:
            len_ = self.buffer.read_int4()
            self.skip(len_)
        elif type_ == 8:  # map
            size: int = self.read_int32(0)
            for i in range(2 * size):
                self.skip_next_field()
        elif type_ == 9:  # list
            size: int = self.read_int32(0)
            for i in range(size):
                self.skip_next_field()
        elif type_ == 10:
            self.skip_to_struct_end()
        elif type_ == 13:
            self.read_head()
            size: int = self.read_int32(0)
            self.skip(size)

    def skip_next_field(self):
        head, _ = self.read_head()
        self._skip_field(head.type)

    def skip_field(self, count: int):
        for i in range(count):
            self.skip_next_field()

    def read_bytes(self, size: int) -> bytearray:
        b = self.buffer.read_bytes(size)
        return b

    def _read_byte(self) -> bytearray:
        """
        一个字节
        :return:
        """
        return self.read_bytes(1)

    def read_uint16(self) -> int:
        return self.buffer.read_int2()

    def _read_int32(self) -> int:
        return self.buffer.read_int4()

    def _read_int64(self) -> int:
        return self.buffer.read_int8()

    def _read_float32(self) -> float:
        return self.buffer.read_float()

    def _read_float64(self) -> float:
        return self.buffer.read_double()

    def skip_to_tag(self, tag: int) -> bool:
        """
        跳转到tag
        :param tag:
        :return:
        """
        while True:
            head, len_ = self.peak_head()
            if tag <= head.tag or head.type == 11:
                return tag == head.tag
            self.skip(len_)
            self._skip_field(head.type)

    def skip_to_struct_end(self) -> None:
        while True:
            head, _ = self.read_head()
            self._skip_field(head.type)
            if head.type == 11:
                return

    def read_byte(self, tag: int) -> Union[bytes, bytearray]:
        if not self.skip_to_tag(tag):
            return bytes([0])
        head, _ = self.read_head()
        if (type_ := head.type) == 12:
            return bytes([0])
        elif type_ == 0:
            return self._read_byte()
        else:
            return bytes([0])

        pass

    def read_bool(self, tag: int) -> bool:
        return self.read_bytes(tag) != 0

    def read_int16(self, tag: int) -> int:
        if not self.skip_to_tag(tag):
            return 0
        head, _ = self.read_head()
        if (type_ := head.type) == 12:
            return 0
        elif type_ == 0:
            return self._read_byte()[0]
        elif type_ == 1:
            return self.read_uint16()
        else:
            return 0

    def read_int32(self, tag: int) -> int:
        if not self.skip_to_tag(tag):
            return 0
        head, _ = self.read_head()
        if (type_ := head.type) == 12:
            return 0
        elif type_ == 0:
            return self._read_byte()[0]
        elif type_ == 1:
            return self.read_uint16()
        elif type_ == 2:
            return self._read_int32()
        else:
            return 0

    def read_int64(self, tag: int) -> int:
        if not self.skip_to_tag(tag):
            return 0
        head, _ = self.read_head()
        if (type_ := head.type) == 12:
            return 0
        elif type_ == 0:
            return self._read_byte()[0]
        elif type_ == 1:
            return self.read_uint16()
        elif type_ == 2:
            return self._read_int32()
        elif type_ == 3:
            return self._read_int64()
        else:
            return 0

    def read_float32(self, tag: int) -> float:
        if not self.skip_to_tag(tag):
            return 0.0
        head, _ = self.read_head()
        if (type_ := head.type) == 12:
            return 0.0
        elif type_ == 4:
            return self._read_float32()
        else:
            return 0.0

    def read_float64(self, tag: int):
        if not self.skip_to_tag(tag):
            return 0.0
        head, _ = self.read_head()
        if (type_ := head.type) == 12:
            return 0.0
        elif type_ == 4:
            return self._read_float32()
        elif type_ == 5:
            return self._read_float64()
        else:
            return 0.0
        pass

    def read_string(self, tag: int):
        if not self.skip_to_tag(tag):
            return ""
        head, _ = self.read_head()
        if (type_ := head.type) == 6:
            return self.read_bytes(self._read_byte()[0]).decode(DEFAULT_ENCODING)
        elif type_ == 7:
            return self.read_bytes(self._read_int32()).decode(DEFAULT_ENCODING)
        else:
            return ""

    # ReadAny Read any type via tag, unsupported JceStruct
    def read_any(self, tag: int) -> Any:
        if not self.skip_to_tag(tag):
            return
        head, _ = self.read_head()
        if (type_ := head.type) == 0:
            return self._read_byte()[0]
        elif type_ == 1:
            return self.read_uint16()
        elif type_ == 2:
            return self._read_int32()
        elif type_ == 3:
            return self._read_int64()
        elif type_ == 4:
            return self._read_float32()
        elif type_ == 5:
            return self._read_float64()
        elif type_ == 6:
            return self.read_bytes(self._read_byte()[0]).decode(DEFAULT_ENCODING)
        elif type_ == 7:
            return self.read_bytes(self._read_int32()).decode(DEFAULT_ENCODING)
        elif type_ == 8:  # map csdn的文档大有问题
            size: int = self.read_int32(0)  # 跳到字典
            m = {}
            for i in range(size):
                k = self.read_any(0)  # 不这么写会出问题
                v = self.read_any(1)
                m[k] = v
            return m
        elif type_ == 9:  # list
            sl = []
            size = self.read_int32(0)
            for i in range(size):
                sl.append(self.read_any(0))
            return sl
        elif type_ == 10:  # obj
            sl = []
            while True:
                head, _ = self.peak_head()
                if head.type == 11 and head.tag == 0:
                    self.read_head()  # 去掉结束标志
                    break
                sl.append(self.read_any(head.tag))
            return sl

        elif type_ == 11:
            return None

        elif type_ == 12:
            return 0
        elif type_ == 13:  # simple list   head len data
            self.read_head()
            return self.read_bytes(self.read_int32(0))
        else:
            return

    def read_any_with_tag(self, tag: int) -> Any:
        """
        同上,但是会是Dict[tag,value]的递归的组合
        e.g. 上一个函数返回[1,2,3] ,这个返回{1:1,2:2,3:3},就是带上了tag
        :param tag:
        :return:
        """
        if not self.skip_to_tag(tag):
            return
        head, _ = self.read_head()
        if (type_ := head.type) == 0:
            return self._read_byte()[0]
        elif type_ == 1:
            return self.read_uint16()
        elif type_ == 2:
            return self._read_int32()
        elif type_ == 3:
            return self._read_int64()
        elif type_ == 4:
            return self._read_float32()
        elif type_ == 5:
            return self._read_float64()
        elif type_ == 6:
            return self.read_bytes(self._read_byte()[0]).decode(DEFAULT_ENCODING)
        elif type_ == 7:
            return self.read_bytes(self._read_int32()).decode(DEFAULT_ENCODING)
        elif type_ == 8:  # map csdn的文档大有问题
            size: int = self.read_int32(0)  # 跳到字典
            m = {}
            for i in range(size):
                k = self.read_any_with_tag(0)  # 不这么写会出问题
                v = self.read_any_with_tag(1)
                m[k] = v
            return m
        elif type_ == 9:  # list
            sl = []
            size = self.read_int32(0)
            for i in range(size):
                sl.append(self.read_any_with_tag(0))
            return sl
        elif type_ == 10:  # obj
            sl = {}
            while True:
                head, _ = self.peak_head()
                if head.type == 11 and head.tag == 0:
                    self.read_head()  # 去掉结束标志
                    break
                sl[head.tag] = self.read_any_with_tag(head.tag)
            return sl

        elif type_ == 11:
            return None

        elif type_ == 12:
            return 0
        elif type_ == 13:  # simple list   head len data
            self.read_head()
            return self.read_bytes(self.read_int32(0))
        else:
            return

    def read_map_f(self, tag: int, func: Callable[[Any, Any], Any]) -> None:
        if not self.skip_to_tag(tag):
            return
        self.read_head()  # 去头就可以吃了
        size = self.read_int32(0)
        for i in range(size):
            k = self.read_any(0)
            v = self.read_any(1)
            if k is not None:
                func(k, v)

    def read_map(self, tag: int) -> dict:
        """返回字典"""
        if not self.skip_to_tag(tag):
            return {}
        self.read_head()
        _dict = {}
        size = self.read_int32(0)
        for i in range(size):
            k = self.read_any(0)
            v = self.read_any(1)
            _dict.update({k: v})

        return _dict

    def read_list(self, type_: Type[IJceStruct], tag: int) -> List[IJceStruct]:
        """
        根据type来实例化 list of type10
        :param type_: IJceStruct的子类 实例化标志
        :param tag:
        :return:
        """
        sl = []
        if not self.skip_to_tag(tag):
            return sl
        head, _ = self.read_head()
        if head.type == 9:
            size = self.read_int32(0)
            for i in range(size):
                data = self.read_object(type_)
                sl.append(data)
            return sl

    def read_object(self, type_: Type[IJceStruct]) -> IJceStruct:
        """
        读取自定义结构,传入一个类 里面必须是type10
        :param type_: IJceStruct的子类
        :return:
        """
        if issubclass(type_, IJceStruct):
            data = type_()
            self.read_head()
            data.read_from(self)
            self.skip_to_struct_end()
            return data

    def read_available(self) -> bytes:
        """
        读取全部缓冲区剩下的
        :return:
        """
        return self.read_bytes(len(self.buffer) - self.buffer.position)



from and_tools import Pack, get_md5


def tlv_head(head, data):
    pack = Pack()
    pack.add_hex(head)
    pack.add_int(len(data), 2)
    pack.add_bin(data)
    return pack.get_bytes()


class TLV:
    def __init__(self, info):
        self.pack = Pack()
        self.info = info

    def T100(self, sso_version, app_id, app_client_version, sigmap):
        self.pack.empty()
        self.pack.add_hex('00 01')
        self.pack.add_int(sso_version)
        self.pack.add_int(app_id)
        self.pack.add_int(self.info.device.app_id)
        self.pack.add_int(app_client_version)
        self.pack.add_int(sigmap)
        _data = self.pack.get_bytes()
        return tlv_head('01 00', _data)

    def T10A(self, T0A: bytes):
        return tlv_head('01 0A', T0A)

    def T116(self, image_type=10):
        self.pack.empty()
        if self.info.device.client_type == 'Watch':
            self.pack.add_hex('00 00 F7 FF 7C 00 01 04 00 00')
        elif self.info.device.client_type == 'QQ':
            self.pack.add_int(image_type, 2)
            self.pack.add_hex('F7 FF 7C 00 01 04 00 01 5F 5E 10 E2')
        else:
            _data = b''
        _data = self.pack.get_bytes()
        return tlv_head('01 16', _data)

    def T143(self, T143: bytes):
        return tlv_head('01 43', T143)

    def T142(self):
        self.pack.empty()
        self.pack.add_body(self.info.device.package_name, 4)
        return tlv_head('01 42', self.pack.get_bytes())

    def T154(self):
        self.pack.empty()
        self.pack.add_int(self.info.seq)
        return tlv_head('01 54', self.pack.get_bytes())

    def T017(self, app_id: int, Uin: int, login_time: int):
        self.pack.empty()
        self.pack.add_hex('00 16 00 01')
        self.pack.add_hex('00 00 06 00')
        self.pack.add_int(app_id)
        self.pack.add_hex('00 00 00 00')
        self.pack.add_int(Uin)
        self.pack.add_hex('00 00 00 00')
        return tlv_head('00 17', self.pack.get_bytes())

    def T141(self):
        self.pack.empty()
        self.pack.add_hex('00 01')
        self.pack.add_body(self.info.device.internet, 2)
        self.pack.add_hex('00 02')
        self.pack.add_body(self.info.device.internet_type, 2)
        _data = self.pack.get_bytes()
        return tlv_head('01 41', _data)

    def T008(self):
        self.pack.empty()
        self.pack.add_hex('00 00 00 00 08 04 00 00')
        return tlv_head('00 08', self.pack.get_bytes())

    def T147(self):
        self.pack.empty()
        self.pack.add_hex('00 00 00 10')
        self.pack.add_body(self.info.device.version, 2, )
        self.pack.add_body(self.info.device.Sig, 2, True)
        _data = self.pack.get_bytes()
        return tlv_head('01 47', _data)

    def T177(self):
        self.pack.empty()
        self.pack.add_hex('01')
        self.pack.add_int(self.info.device.build_time)
        self.pack.add_body(self.info.device.sdk_version, 2)
        _data = self.pack.get_bytes()
        return tlv_head('01 77', _data)

    def T187(self):
        Mac = get_md5(self.info.device.Mac)
        self.pack.empty()
        self.pack.add_body(Mac, 2)
        _data = self.pack.get_bytes()
        return tlv_head('01 87', _data)

    def T188(self):
        _app_id = get_md5(str(self.info.device.app_id))

        self.pack.empty()
        self.pack.add_body(_app_id, 2)
        _data = self.pack.get_bytes()
        return tlv_head('01 88', _data)

    def T202(self):
        Bssid = get_md5(self.info.device.Bssid)
        self.pack.empty()
        self.pack.add_body(Bssid, 2)
        self.pack.add_body('<unknown ssid>', 2)
        _data = self.pack.get_bytes()
        return tlv_head('02 02', _data)

    def T511(self):
        """
            office.qq.com
            qun.qq.comgamecenter.qq.comdocs.qq.commail.qq.com	ti.qq.com
            vip.qq.com
            tenpay.comqqweb.qq.comqzone.qq.com
            mma.qq.comgame.qq.comopenmobile.qq.comconnect.qq.com"""

        domain = [
            'office.qq.com',
            'qun.qq.com',
            'gamecenter.qq.com',
            # 'graph.qq.com',
            # 'docs.qq.com',
            'mail.qq.com',
            'ti.qq.com',
            'vip.qq.com',
            # 'tenpay.com',
            'qqweb.qq.com',
            'qzone.qq.com',
            'mma.qq.com',
            'game.qq.com',
            # 'openmobile.qq.com',
            'connect.qq.com',
            'accounts.qq.com',
            # 'weishi.qq.com',

        ]
        self.pack.empty()
        self.pack.add_int(len(domain), 2)  # 数量
        for domain_result in domain:
            self.pack.add_hex('01')
            self.pack.add_body(domain_result, 2)

        return tlv_head('05 11', self.pack.get_bytes())



# -*- coding: utf-8 -*-
import abc
from typing import Union
from pydantic import BaseModel

from .typing import JceReader


class IJceStruct(BaseModel, abc.ABC):
    """
    jce结构体的基础
    """

    @abc.abstractmethod
    def read_from(self, reader: JceReader) -> None:
        raise NotImplementedError

    @abc.abstractmethod
    def to_bytes(self) -> Union[bytes, bytearray]:
        raise NotImplementedError

    def to_dict(self) -> dict:
        return self.__dict__



from random import randint

from and_tools import Pack, get_random_bin, TEA
from and_tools.Jce_b import JceWriter


def pack_(info, data, types, encryption, sso_seq=None, token=False):
    """组包
    Types:包类型  通常是10 和11
    encryption  加密方式   2 不需要token  1需要token
    sso_seq 包序号
    """
    pack = Pack()
    pack.add_int(types)
    pack.add_bytes(encryption)
    if not info.uin:
        Uin_bytes = bytes.fromhex('30')
    else:
        Uin_bytes = info.uin.encode('utf-8')

    if token:
        D2 = info.UN_Tlv_list.D2_T143
        pack.add_int(len(D2) + 4)
        pack.add_bin(D2)

    if sso_seq is not None:
        pack.add_int(sso_seq)
    pack.add_bytes(0)

    pack.add_int(len(Uin_bytes) + 4)
    pack.add_bin(Uin_bytes)
    pack.add_bin(data)
    bytes_temp = pack.get_bytes()

    pack.empty()
    pack.add_int(len(bytes_temp) + 4)
    pack.add_bin(bytes_temp)
    return pack.get_bytes()


def pack_head_no_token(info, sBuffer, serviceCmd, sServantName=None, sFuncName=None):
    """照抄的旧源码"""
    # struct com.qq.taf;

    if sServantName is not None:
        """jce部分"""
        iRequestId = randint(110000000, 999999999)
        jce = JceWriter()
        jce.write_int32(3, 1)  # iVersion
        jce.write_int32(0, 2)  # cPacketType
        jce.write_int32(0, 3)  # iMessageType
        jce.write_int64(iRequestId, 4)
        jce.write_string(sServantName, 5)
        jce.write_string(sFuncName, 6)
        jce.write_bytes(sBuffer, 7)
        jce.write_int32(0, 8)  # iTimeout
        _data = jce.bytes()
        _data = _data + bytes.fromhex('98 0C A8 0C')  # 后面的两个空的
    else:
        _data = sBuffer

    pack = Pack()
    pack.add_int(len(serviceCmd) + 4)
    pack.add_bin(bytes(serviceCmd, 'utf-8'))
    pack.add_hex('00 00 00 08')
    pack.add_bin(get_random_bin(4))
    pack.add_hex('00 00 00 04')
    _data_temp = pack.get_bytes()

    pack.empty()
    pack.add_int(len(_data_temp) + 4)
    pack.add_bin(_data_temp)
    _data_temp = pack.get_bytes()

    pack.empty()
    pack.add_bin(_data_temp)
    pack.add_int(len(_data) + 4)
    pack.add_bin(_data)
    _data = pack.get_bytes()
    _data = TEA.encrypt(_data, info.share_key)

    return _data


def pack_head(info, data, cmd):
    """包头"""
    TokenA4 = info.UN_Tlv_list.TGT_T10A
    if TokenA4 is None:
        TokenA4 = b''

    pack = Pack()
    pack.add_int(info.seq)
    pack.add_int(info.device.app_id)
    pack.add_int(info.device.app_id)

    pack.add_hex('01 00 00 00')
    pack.add_hex('00 00 00 00')
    pack.add_hex('00 00 01 00')
    pack.add_int(len(TokenA4) + 4)
    pack.add_bin(TokenA4)
    pack.add_int(len(cmd) + 4)
    pack.add_bin(bytes(cmd, 'utf-8'))
    pack.add_hex('00 00 00 08')
    pack.add_bin(get_random_bin(4))

    pack.add_body(info.device.Imei, 4, add_len=4)
    pack.add_hex('00 00 00 04')

    pack.add_body(info.device.var, 2, add_len=2)

    bytes_temp = pack.get_bytes()

    pack.empty()
    pack.add_int(len(bytes_temp) + 4)
    pack.add_bin(bytes_temp)
    bytes_temp = pack.get_bytes()

    pack.empty()
    pack.add_bin(bytes_temp)
    pack.add_int(len(data) + 4)
    pack.add_bin(data)
    bytes_temp = pack.get_bytes()
    bytes_temp = TEA.encrypt(bytes_temp, info.share_key)
    return bytes_temp


def pack_head_login(info, Cmd, Buffer):
    """返回前面的头,后面的单独写在组包的函数里面
    01 8C
    1F 41
    08 12
    """
    pack = Pack()
    pack.add_int(info.seq)
    pack.add_int(info.device.app_id)
    pack.add_int(info.device.app_id)
    pack.add_hex('01 00 00 00 00 00 00 00 00 00 01 00')
    pack.add_body(info.UN_Tlv_list.TGT_T10A, 4, add_len=4)
    pack.add_body(Cmd, 4, add_len=4)
    pack.add_body(get_random_bin(4), length=4, add_len=4)  # MsgCookies
    pack.add_body(info.device.Imei, 4, add_len=4)
    pack.add_hex('00 00 00 04')
    pack.add_body(info.device.var, 2, add_len=2)
    bytes_temp = pack.get_bytes()

    pack.empty()
    pack.add_body(bytes_temp, 4, add_len=4)
    pack.add_body(Buffer, 4, add_len=4)
    data = pack.get_bytes()
    data = TEA.encrypt(data, '00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00')
    return data


def PackHead_QMF(info, serviceCmd, Upstream):
    """缝补补,又一年"""
    pack = Pack()
    pack.add_int(len(serviceCmd) + 4)
    pack.add_bin(bytes(serviceCmd, 'utf-8'))
    pack.add_hex('00 00 00 08')
    pack.add_bin(get_random_bin(4))
    pack.add_hex('00 00 00 04')
    _data_temp = pack.get_bytes()

    pack.empty()
    pack.add_int(len(_data_temp) + 4)
    pack.add_bin(_data_temp)
    _data_temp = pack.get_bytes()

    pack.empty()
    pack.add_bin(_data_temp)
    pack.add_int(len(Upstream) + 4)
    pack.add_bin(Upstream)
    _data = pack.get_bytes()

    Buffer = TEA.encrypt(_data, info.share_key)

    Buffer = pack_(info, Buffer, types=11, encryption=1, sso_seq=info.seq)

    return Buffer



class UnPack:
    """解包"""

    def __init__(self, data):
        if isinstance(data, str):
            self._byte_data = bytearray.fromhex(data.replace(' ', ''))
        else:
            self._byte_data = data

    def _get_bytes(self, length: int) -> bytes:
        res = self._byte_data[:length]
        self._byte_data = self._byte_data[length:]
        return res

    def get_int(self, length=4):
        """取整数"""
        res = self._get_bytes(length)
        return int.from_bytes(res, 'big')

    def get_long(self):
        """取长整数"""
        res = self._get_bytes(8)
        return int.from_bytes(res, 'big')

    def get_byte(self):
        """取字节"""
        res = self._get_bytes(1)
        return int.from_bytes(res, 'big', signed=True)

    def get_bin(self, length):
        """取字节集"""
        res = self._get_bytes(length)
        return res

    def get_short(self):
        """取短整数"""
        res = self._get_bytes(2)
        return int.from_bytes(res, 'big')

    def get_len(self):
        """取长度"""
        return len(self._byte_data)

    def get_all(self, _hex=False):
        """取全部"""
        res = self._byte_data[:]
        if _hex:
            res = ' '.join(['{:02x}'.format(byte) for byte in res])
        return res



# -*- coding: utf-8 -*-
"""
jce field 的头部 tag type
"""
from typing import Optional


class HeadData(object):
    """
    ttlv的头部信息
    tag1 type [tag2] length value需要处理
    """

    def __init__(self, tag: Optional[int] = 0, type_: Optional[int] = 0):
        if tag < 256:
            self.tag = tag
        else:
            raise ValueError("tag can't greater than 255")
        self.type = type_

    def __str__(self):
        return '{tag: %d, type: %d}' % (self.tag, self.type)

    def __repr__(self):
        return '{tag: %d, type: %d}' % (self.tag, self.type)

    def as_dict(self):
        return {"tag": self.tag, "type": self.type}

    def as_bytes(self) -> bytes:
        """
        转化为二进制用于写入
        :return: 要么1字节 要么2字节
        """
        if self.tag < 15:
            return bytes([self.tag << 4 | self.type])
        else:
            return bytes([15 | self.type]) + bytes([self.tag])

    def clear(self):
        self.tag = 0
        self.type = 0



from typing import Tuple, List

from and_tools.Jce import ByteBuffer
from and_tools.Jce.exception import JceDecodeException
from and_tools.Jce.struct import JceStructStatics, JceStruct


class HeadData(object):
    tag: int = 0
    type: int = 0

    def __init__(self):
        self.tag = 0
        self.type = 0

    def __str__(self):
        return '{tag: %d, type: %d}' % (self.tag, self.type)

    def __repr__(self):
        return '{tag: %d, type: %d}' % (self.tag, self.type)

    def clear(self):
        self.tag = 0
        self.type = 0


def read_head(byte_buffer: ByteBuffer) -> Tuple[HeadData, int]:
    head_data = HeadData()
    b = byte_buffer.get()
    head_data.type = b & 0x0F  # 低4位位类型
    head_data.tag = (b & 0xF0) >> 4  # 高4位为tag,
    if head_data.tag != 15:  # 如果tag为15 则下一个字段为tag
        return (head_data, 1)
    else:
        head_data.tag = byte_buffer.get() & 0xFF
        return (head_data, 2)


class JceInputStream(object):
    _bs: ByteBuffer = None
    encoding = 'utf-8'

    def __init__(self, bs, i=0):
        self.encoding = 'utf-8'
        if isinstance(bs, ByteBuffer):
            self._bs = bs
        elif isinstance(bs, (bytearray, bytes)):
            self._bs = ByteBuffer(bs)
            self._bs.position = i
        else:
            raise TypeError("'bs'参数必须是 bytes、bytesarray 或 ByteBuffer")

    def read_head(self):
        return read_head(self._bs)

    def peak_head(self):
        return read_head(self._bs.duplicate())

    def skip(self, i: int):
        self._bs.position = self._bs.position + i

    def skip_to_struct_end(self):
        head_data, _ = self.read_head()
        self.skip_field(head_data.type)
        while (head_data.type != 11):
            head_data, _ = self.read_head()
            self.skip_field(head_data.type)

    def skip_field(self, field_type=None):
        if not field_type:
            head_data, _ = self.read_head()
            field_type = head_data.type
        i = 0
        read_value = None
        if field_type == 0:
            self.skip(1)
        elif field_type == 1:
            self.skip(2)
        elif field_type == 2:
            self.skip(4)
        elif field_type == 3:
            self.skip(8)
        elif field_type == 4:
            self.skip(4)
        elif field_type == 5:
            self.skip(8)
        elif field_type == 6:
            i = self._bs.get()
            if i < 0:
                i += 256
            self.skip(i)
        elif field_type == 7:
            i = self._bs.get_int4()
            self.skip(i)
        elif field_type == 8:
            read_value = self._read_int(0, 0, True)
            while i < read_value * 2:
                self.skip_field()
                i += 1
        elif field_type == 9:
            read_value = self._read_int(0, 0, True)
            while i < read_value:
                self.skip_field()
                i += i
        elif field_type == 10:
            self.skip_to_struct_end()
        elif field_type == 11 or field_type == 12:
            return
        elif field_type == 13:
            head_data, _ = self.read_head()
            if head_data.type != 0:
                raise JceDecodeException(
                    "skipField with invalid type, type value: " + field_type + ", " + head_data.type)
            i = self._read_int(0, 0, True)
            self.skip(i)
        else:
            raise JceDecodeException("invalid type.")

    def skip_to_tag(self, tag: int) -> bool:
        try:
            while True:
                head_data, length = self.peak_head()
                if tag > head_data.tag and head_data.type != 0x0B:
                    self.skip(length)
                    self.skip_field(head_data.type)
                else:
                    break
            if head_data.type == 0X0B or tag != head_data.tag:
                return False
            return True
        except (JceDecodeException, BufferError):
            return False

    def re_init(self):
        self.encoding = 'utf-8'
        self._bs.clear()

    def get_tags(self) -> List[int]:
        position = self._bs.position
        # self.re_init()
        tags = []
        while True:
            try:
                head_data, _ = self.read_head()
                tags.append(head_data.tag)
                self.skip_field(head_data.type)
            except:
                print('exception occured in position: %d, quit' % (self._bs.position))
                break
        self._bs.position = position
        return tags

    def _read_bool(self, b: bool, tag: int, is_require: bool) -> bool:
        c = self._read_int(0, tag, is_require)
        return c != 0

    def _read_int(self, c: int, tag: int, is_require) -> int:
        if self.skip_to_tag(tag):
            head_data, _ = self.read_head()
            if head_data.type == 12:
                c = 0
            elif head_data.type == 0:
                c = self._bs.get()
            elif head_data.type == 1:
                c = self._bs.get_int2()
            elif head_data.type == 2:
                c = self._bs.get_int4()
            elif head_data.type == 3:
                c = self._bs.get_int8()
            else:
                raise JceDecodeException("type mismatch.")
        elif is_require:
            raise JceDecodeException("require field not exist.")
        return c

    def _read_float(self, n: float, tag: int, is_require: bool) -> float:
        if self.skip_to_tag(tag):
            head_data, _ = self.read_head()
            if head_data.type == 12:
                n = 0.0
            elif head_data.type == 4:
                n = self._bs.get_float()
            if head_data.type == 5:
                n = self._bs.get_double()
            else:
                raise JceDecodeException("type mismatch.")
        elif is_require:
            raise JceDecodeException("require field not exist.")
        return n

    def _read_string(self, s: str, tag: int, is_require: bool) -> str:
        """_读取字符串  """
        if self.skip_to_tag(tag):
            head_data, _ = self.read_head()
            if head_data.type == 6:
                length = self._bs.get()
                if length < 0:
                    length += 256
                ss = self._bs.get_bytes(length)

                try:

                    s = ss.decode(self.encoding)

                except UnicodeDecodeError:
                    s = ss.decode(errors='replace')
            elif head_data.type == 7:
                length = self._bs.get_int4()
                if length > JceStructStatics.JCE_MAX_STRING_LENGTH or length < 0:
                    raise JceDecodeException("字符串太长： " + len)
                ss = self._bs.get_bytes(length)
                try:
                    s = ss.decode(self.encoding)
                except UnicodeDecodeError:
                    s = ss.decode(errors='replace')
            else:
                raise JceDecodeException("类型不匹配。")
        elif is_require:
            raise JceDecodeException("要求字段不存在。")
        return s

    def _read_struct(self, o: JceStruct, tag: int, is_require: bool) -> JceStruct:
        ref = None
        if self.skip_to_tag(tag):
            ref = o
            head_data, _ = self.read_head()
            if head_data.type != 10:
                raise JceDecodeException("type mismatch.")
            ref.read_from(self)
            self.skip_to_struct_end()
        elif is_require:
            raise JceDecodeException("require field not exist.")
        return ref

    def _read_list(self, mt, tag: int, is_require: bool) -> list:
        if self.skip_to_tag(tag):
            head_data, _ = self.read_head()
            if head_data.type == 9:
                size = self._read_int(0, 0, True)
                if size < 0:
                    raise JceDecodeException("size invalid: " + size)
                lr = []
                for i in range(size):
                    t = self.read_current(True)
                    lr.append(t)
                return lr
            raise JceDecodeException("type mismatch.")
        elif is_require:
            raise JceDecodeException("require field not exist.")
        return None

    def _read_map(self, m: dict, tag: int, is_require: bool):
        mr = {}
        if self.skip_to_tag(tag):
            head_data, _ = self.read_head()
            if head_data.type == 8:
                size = self._read_int(0, 0, True)
                if size < 0:
                    raise JceDecodeException("size invalid: " + size)
                for i in range(size):
                    k = self.read_current(True)
                    v = self.read_current(True)
                    mr[k] = v
            else:
                raise JceDecodeException("type mismatch.")
        elif is_require:
            raise JceDecodeException("require field not exist.")
        return mr

    def _read_simple_list(self, l, tag: int, is_require: bool):
        """_读取简单列表"""
        lr = b''
        if self.skip_to_tag(tag):
            head_data, _ = self.read_head()
            if head_data.type == 13:
                hh, _ = self.read_head()
                if hh.type != 0:
                    raise JceDecodeException(
                        "type mismatch, tag: " + tag + ", type: " + head_data.type + ", " + hh.type)
                size = self._read_int(0, 0, True)
                if size < 0:
                    raise JceDecodeException(
                        "invalid size, tag: " + tag + ", type: " + head_data.type + ", " + hh.type + ", size: " + size)
                lr = self._bs.get_bytes(size)
            else:
                raise JceDecodeException("类型不匹配。")
        elif is_require:
            raise JceDecodeException("要求字段不存在。")
        return lr.hex()  # 返回hex

    def read(self, o, tag: int, is_require: bool):
        if isinstance(o, bool):
            return self._read_bool(o, tag, is_require)
        if isinstance(o, int):
            return self._read_int(o, tag, is_require)
        if isinstance(o, float):
            return self._read_float(o, tag, is_require)
        if isinstance(o, str):
            return self._read_string(o, tag, is_require)
        if isinstance(o, list):
            return self._read_list(o, tag, True)
        if isinstance(o, dict):
            return self._read_map(o, tag, True)
        raise JceDecodeException("read object error: unsupport type.")

    def read_current(self, is_require: bool):
        """读取当前"""
        head_data, _ = self.peak_head()
        if head_data.type in (0, 1, 2, 3):
            return self._read_int(0, head_data.tag, is_require)
        elif head_data.type in (4, 5):
            return self._read_float(0.0, head_data.tag, is_require)
        elif head_data.type in (6, 7):
            return self._read_string('', head_data.tag, is_require)
        elif head_data.type == 8:
            return self._read_map({}, head_data.tag, is_require)
        elif head_data.type == 9:
            return self._read_list([], head_data.tag, is_require)
        elif head_data.type == 10:
            return self._read_struct(JceStruct(), head_data.tag, is_require)
        elif head_data.type == 11:
            self.read_head()
            return None
        elif head_data.type == 12:
            # ZERO_TAG
            self.read_head()
            return 0
        elif head_data.type == 13:
            # BYTES 字节流
            return self._read_simple_list(b'', head_data.tag, is_require)
        else:
            raise JceDecodeException("读取对象错误：类型不受支持。")



# -*- coding: utf-8 -*-
import struct
from copy import deepcopy
from typing import Optional, Union


class ByteBuffer:
    """
    字节缓冲区
    """

    # _bytes = None
    # _position = 0

    def __init__(self, bs: Optional[Union[bytes, bytearray]] = None):
        if bs is None:
            self._bytes = bytearray()
        elif isinstance(bs, bytearray):
            self._bytes = bs
        elif isinstance(bs, bytes):
            self._bytes = bytearray(bs)
        else:
            raise TypeError("'buffer' argument must be bytes or bytesarray")
        self._position = 0

    def __len__(self):
        return len(self._bytes)

    @property
    def bytes(self) -> bytearray:
        """
        返回自己的全部数据
        :return:
        """
        return self._bytes

    @property
    def position(self):
        """
        位置指针 当前读取的字节数 读取到第几个字节了
        :return:
        """
        return self._position

    @position.setter
    def position(self, value):
        """
        设置读取指针
        :param value:
        :return:
        """
        if not isinstance(value, int):
            raise TypeError("'position' attribute must be a integer")
        elif value < 0:
            raise ValueError("'position' attribute must be a positive number")
        elif value > len(self._bytes):
            raise ValueError('position out of index range')
        else:
            self._position = value

    def read(self) -> int:
        """
        读取一个字节并返回 指针+1
        :return:
        """
        if self._position >= len(self._bytes):
            raise BufferError('reached end of bytes')
        b = self._bytes[self._position]
        self._position += 1
        return b

    def read_bytes(self, size: int) -> bytearray:
        """
        读取接下来的size个字节 指针向后面移动size
        :param size:
        :return:
        """
        if size < 0:
            raise ValueError("'size' attribute must be a positive number")
        if self._position > len(self._bytes):
            raise BufferError('reached end of bytes')
        if self.position + size > len(self._bytes):
            raise BufferError('reached end of bytes')
        b = self._bytes[self.position:self.position + size]
        self.position = self.position + size
        return b

    def read_int2(self) -> int:
        """
        读取一个jce中的int2类型
        :return:
        """
        b = self.read_bytes(2)
        return struct.unpack('>h', b)[0]  # 解包出来是个元组 得像这样加料

    def read_uint2(self) -> int:
        """
        读取一个jce中的uint2类型
        :return:
        """
        b = self.read_bytes(2)
        return struct.unpack('>H', b)[0]  # 解包出来是个元组 得像这样加料

    def read_int4(self) -> int:
        """
        读取一个jce中的int4类型
        :return:
        """
        b = self.read_bytes(4)
        return struct.unpack('>i', b)[0]

    def read_uint4(self) -> int:
        """
        读取一个jce中的uint4类型
        :return:
        """
        b = self.read_bytes(4)
        return struct.unpack('>I', b)[0]

    def read_int8(self) -> int:
        """
        读取一个jce中的int8类型
        :return:
        """
        b = self.read_bytes(8)
        return struct.unpack('>q', b)[0]

    def read_uint8(self) -> int:
        """
        读取一个jce中的int8类型
        :return:
        """
        b = self.read_bytes(8)
        return struct.unpack('>Q', b)[0]

    def read_float(self) -> float:
        """
        读取一个jce中的float类型 4字节
        :return:
        """
        b = self.read_bytes(4)
        return struct.unpack('>f', b)[0]

    def read_double(self) -> float:
        """
        读取一个jce中的double类型 8字节
        :return:
        """
        b = self.read_bytes(8)
        return struct.unpack('>d', b)[0]

    def write_bytes(self, data: Union["bytes", bytearray]) -> None:
        """
        写入一个字节流
        :param data:
        :return:
        """
        self._bytes.extend(data)
        self._position += len(data)

    def write_hex(self, hexstr: str) -> None:
        str_bytes: str = hexstr.strip()
        pkt = bytes.fromhex(str_bytes)
        self.write_bytes(pkt)

    def write_int2(self, num) -> None:
        pkt = struct.pack(">h", num)
        self.write_bytes(pkt)

    def write_uint2(self, num) -> None:
        pkt = struct.pack(">H", num)
        self.write_bytes(pkt)

    def write_int4(self, num) -> None:
        pkt = struct.pack(">i", num)
        self.write_bytes(pkt)

    def write_uint4(self, num) -> None:
        pkt = struct.pack(">I", num)
        self.write_bytes(pkt)

    def write_int8(self, num) -> None:
        pkt = struct.pack(">q", num)
        self.write_bytes(pkt)

    def write_uint8(self, num) -> None:
        pkt = struct.pack(">Q", num)
        self.write_bytes(pkt)

    def write_float(self, num):
        pkt = struct.pack(">f", num)
        self.write_bytes(pkt)

    def write_double(self, num) -> None:
        pkt = struct.pack(">d", num)
        self.write_bytes(pkt)

    def copy(self) -> "ByteBuffer":
        """
        返回自己的一份深拷贝
        :return:
        """
        return deepcopy(self)

    def seek(self, position: int) -> None:
        """
        重新定位指针
        :param position:
        :return:
        """
        self.position = position

    def clear(self) -> None:
        """
        指针重新回到0
        :return:
        """
        self._position = 0



# -*- coding: utf-8 -*-
from typing import TypeVar, TYPE_CHECKING

if TYPE_CHECKING:
    from .reader import JceReader
    from .writer import JceWriter
    from .struct import IJceStruct
    from .buffer import ByteBuffer
    from .head import HeadData

JceReader = TypeVar("JceReader", bound="JceReader")

JceWriter = TypeVar("JceWriter", bound="JceWriter")

IJceStruct = TypeVar("IJceStruct", bound="IJceStruct")

ByteBuffer = TypeVar("ByteBuffer", bound="ByteBuffer")

HeadData = TypeVar("HeadData", bound="HeadData")



import hashlib
import random


def hex_format(_hex):
    """Hex格式化"""
    Hex = ' '.join(_hex[i:i + 2] for i in range(0, len(_hex), 2))
    return Hex


def get_random_bin(_len):
    byte = b""
    for _ in range(_len):
        byte += bytes([random.randint(0, 255)])
    return byte


def get_md5(byte_content):
    if isinstance(byte_content, str):
        byte_content = byte_content.encode('utf-8')
    md5_object = hashlib.md5()
    md5_object.update(byte_content)
    return md5_object.digest()



from and_tools.Jce_b import JceReader


def un_jce_head(data) -> bytes:
    jce = JceReader(data)
    jce.read_int32(1)
    jce.read_int32(2)
    jce.read_int32(3)
    jce.read_int64(4)
    jce.read_string(5)
    jce.read_string(6)
    data = jce.read_any(7)
    return data


def un_jce_head_2(data) -> bytes:
    """Map"""
    jce = JceReader(data)
    jce.skip(1)  # ReadType
    jce.skip(2)  # ReadShort
    jce.read_string(0)
    jce.skip(1)  # ReadType
    jce.skip(2)  # ReadShort
    jce.read_string(0)
    data = jce.read_any(1)
    return data



import zlib

from and_tools.Jce import JceInputStream, JceStruct
from and_tools.Jce_b import JceWriter
from .SvcReqGetDevLoginInfo import SvcReqGetDevLoginInfo
from ...utils import pack_head_no_token, pack_, un_jce_head, un_jce_head_2


def GetDevLoginInfo_profile(info, item: SvcReqGetDevLoginInfo):
    """获取设备登录信息"""
    _buffer = item.to_bytes()
    _buffer = JceWriter().write_jce_struct(_buffer, 0)
    _buffer = JceWriter().write_map({'SvcReqGetDevLoginInfo': _buffer}, 0)
    _buffer = pack_head_no_token(info, _buffer, 'StatSvc.GetDevLoginInfo', 'StatSvc', 'SvcReqGetDevLoginInfo')
    _buffer = pack_(info, _buffer, types=11, encryption=1, sso_seq=info.seq)
    return _buffer


def GetDevLoginInfo_res(data):
    if data[0] == 120:
        data = zlib.decompress(data)
    data = un_jce_head(data)
    data = un_jce_head_2(data)
    stream = JceInputStream(data)
    s = JceStruct()
    s.read_from(stream)
    return s.to_json()



from enum import IntEnum, unique
import os


class ProtoError(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return repr(self.msg)


@unique
class ProtoFieldType(IntEnum):
    VARINT = 0
    INT64 = 1
    STRING = 2
    GROUPSTART = 3
    GROUPEND = 4
    INT32 = 5
    ERROR1 = 6
    ERROR2 = 7


class ProtoField:
    def __init__(self, idx, type, val):
        self.idx = idx
        self.type = type
        self.val = val

    def isAsciiStr(self):
        if type(self.val) != bytes:
            return False

        for b in self.val:
            if b < 0x20 or b > 0x7E:
                return False
        return True

    def __str__(self):
        if (
                (self.type == ProtoFieldType.INT32)
                or (self.type == ProtoFieldType.INT64)
                or (self.type == ProtoFieldType.VARINT)
        ):
            return "%d(%s): %d" % (self.idx, self.type.name, self.val)
        elif self.type == ProtoFieldType.STRING:
            if self.isAsciiStr():  # self.val.isalnum()
                return '%d(%s): "%s"' % (
                    self.idx,
                    self.type.name,
                    self.val.decode("ascii"),
                )
            else:
                return '%d(%s): h"%s"' % (self.idx, self.type.name, self.val.hex())
        elif (self.type == ProtoFieldType.GROUPSTART) or (
                self.type == ProtoFieldType.GROUPEND
        ):
            return "%d(%s): %s" % (self.idx, self.type.name, self.val)
        else:
            return "%d(%s): %s" % (self.idx, self.type.name, self.val)


class ProtoReader:
    def __init__(self, data):
        self.data = data
        self.pos = 0

    def seek(self, pos):
        self.pos = pos

    def isRemain(self, length):
        return self.pos + length <= len(self.data)

    def read0(self):
        assert self.isRemain(1)
        ret = self.data[self.pos]
        self.pos += 1
        return ret & 0xFF

    def read(self, length):
        assert self.isRemain(length)
        ret = self.data[self.pos: self.pos + length]
        self.pos += length
        return ret

    def readInt32(self):
        return int.from_bytes(self.read(4), byteorder="little", signed=False)

    def readInt64(self):
        return int.from_bytes(self.read(8), byteorder="little", signed=False)

    def readVarint(self):
        vint = 0
        n = 0
        while True:
            byte = self.read0()
            vint |= (byte & 0x7F) << (7 * n)
            if byte < 0x80:
                break
            n += 1

        return vint

    def readString(self):
        len = self.readVarint()
        return self.read(len)


class ProtoWriter:
    def __init__(self):
        self.data = bytearray()

    def write0(self, byte):
        self.data.append(byte & 0xFF)

    def write(self, bytes):
        self.data.extend(bytes)

    def writeInt32(self, int32):
        bs = int32.to_bytes(4, byteorder="little", signed=False)
        self.write(bs)

    def writeInt64(self, int64):
        bs = int64.to_bytes(8, byteorder="little", signed=False)
        self.write(bs)

    def writeVarint(self, vint):
        vint = vint & 0xFFFFFFFF
        while vint > 0x80:
            self.write0((vint & 0x7F) | 0x80)
            vint >>= 7
        self.write0(vint & 0x7F)

    def writeString(self, bytes):
        self.writeVarint(len(bytes))
        self.write(bytes)

    def toBytes(self):
        return bytes(self.data)


class ProtoBuf:
    def __init__(self, data=None):
        self.fields = []
        if data != None:
            if type(data) != bytes and type(data) != dict:
                raise ProtoError("unsupport type(%s) to protobuf" % (type(data)))

            if (type(data) == bytes) and (len(data) > 0):
                self.__parseBuf(data)
            elif (type(data) == dict) and (len(data) > 0):
                self.__parseDict(data)

    def __getitem__(self, idx):
        pf = self.get(int(idx))
        if pf == None:
            return None
        if pf.type != ProtoFieldType.STRING:
            return pf.val
        if type(idx) != int:
            return pf.val
        if pf.val == None:
            return None
        if pf.isAsciiStr():
            return pf.val.decode("utf-8")
        return ProtoBuf(pf.val)

    def __parseBuf(self, bytes):
        reader = ProtoReader(bytes)
        while reader.isRemain(1):
            key = reader.readVarint()
            field_type = ProtoFieldType(key & 0x7)
            field_idx = key >> 3
            if field_idx == 0:
                break
            if field_type == ProtoFieldType.INT32:
                self.put(ProtoField(field_idx, field_type, reader.readInt32()))
            elif field_type == ProtoFieldType.INT64:
                self.put(ProtoField(field_idx, field_type, reader.readInt64()))
            elif field_type == ProtoFieldType.VARINT:
                self.put(ProtoField(field_idx, field_type, reader.readVarint()))
            elif field_type == ProtoFieldType.STRING:
                self.put(ProtoField(field_idx, field_type, reader.readString()))
            elif field_type == ProtoFieldType.GROUPEND:
                pass
            else:
                raise ProtoError(
                    "parse protobuf error, unexpected field type: %s"
                    % (field_type.name)
                )

    def toBuf(self):
        writer = ProtoWriter()
        for field in self.fields:
            key = (field.idx << 3) | (field.type & 7)
            writer.writeVarint(key)
            if field.type == ProtoFieldType.INT32:
                writer.writeInt32(field.val)
            elif field.type == ProtoFieldType.INT64:
                writer.writeInt64(field.val)
            elif field.type == ProtoFieldType.VARINT:
                writer.writeVarint(field.val)
            elif field.type == ProtoFieldType.STRING:
                writer.writeString(field.val)
            else:
                raise ProtoError(
                    "encode to protobuf error, unexpected field type: %s"
                    % (field.type.name)
                )
        return writer.toBytes()

    def dump(self):
        for field in self.fields:
            print(field)

    def getList(self, idx):
        return [field for field in self.fields if field.idx == idx]

    def get(self, idx):
        for field in self.fields:
            if field.idx == idx:
                return field
        return None

    def getInt(self, idx):
        pf = self.get(idx)
        if pf == None:
            return 0
        if (
                (pf.type == ProtoFieldType.INT32)
                or (pf.type == ProtoFieldType.INT64)
                or (pf.type == ProtoFieldType.VARINT)
        ):
            return pf.val
        raise ProtoError("getInt(%d) -> %s" % (idx, pf.type))

    def getBytes(self, idx):
        pf = self.get(idx)
        if pf == None:
            return None
        if pf.type == ProtoFieldType.STRING:
            return pf.val
        raise ProtoError("getBytes(%d) -> %s" % (idx, pf.type))

    def getUtf8(self, idx):
        bs = self.getBytes(idx)
        if bs == None:
            return None
        return bs.decode("utf-8")

    def getProtoBuf(self, idx):
        bs = self.getBytes(idx)
        if bs == None:
            return None
        return ProtoBuf(bs)

    def put(self, field: ProtoField):
        self.fields.append(field)

    def putInt32(self, idx, int32):
        self.put(ProtoField(idx, ProtoFieldType.INT32, int32))

    def putInt64(self, idx, int64):
        self.put(ProtoField(idx, ProtoFieldType.INT64, int64))

    def putVarint(self, idx, vint):
        self.put(ProtoField(idx, ProtoFieldType.VARINT, vint))

    def putBytes(self, idx, data):
        self.put(ProtoField(idx, ProtoFieldType.STRING, data))

    def putUtf8(self, idx, data):
        self.put(ProtoField(idx, ProtoFieldType.STRING, data.encode("utf-8")))

    def putProtoBuf(self, idx, data):
        self.put(ProtoField(idx, ProtoFieldType.STRING, data.toBuf()))

    def __parseDict(self, data):
        """
        Convert dict object to ProtoBuf object
        """
        for k, v in data.items():
            if isinstance(v, int):
                self.putVarint(k, v)
            elif isinstance(v, str):
                self.putUtf8(k, v)
            elif isinstance(v, bytes):
                self.putBytes(k, v)
            elif isinstance(v, dict):
                self.putProtoBuf(k, ProtoBuf(v))
            else:
                raise ProtoError("unsupport type(%s) to protobuf" % (type(v)))

    def toDict(self, out):
        """
        Convert ProtoBuf object to dict object
        """
        for k, v in out.items():
            if isinstance(v, int):
                out[k] = self.getInt(k)
            elif isinstance(v, str):
                out[k] = self.getUtf8(k)
            elif isinstance(v, bytes):
                out[k] = self.getBytes(k)
            elif isinstance(v, dict):
                out[k] = self.getProtoBuf(k).toDict(v)
            else:
                raise ProtoError("unsupport type(%s) to protobuf" % (type(v)))
        return out

    def toDictAuto(self):
        """Automatic conversion to dict"""
        result = {}
        for field in self.fields:
            key = field.idx
            if field.type in (ProtoFieldType.INT32, ProtoFieldType.INT64, ProtoFieldType.VARINT):
                result[key] = field.val
            elif field.type == ProtoFieldType.STRING:
                if field.isAsciiStr():
                    result[key] = field.val.decode("utf-8")
                else:
                    value = ProtoBuf(field.val).toDictAuto()
                    if not value:
                        value = field.val
                    result[key] = value

        return result


def parse(path):
    """
    Parse proto file or hex string of proto bytes, then print
    """
    if not os.path.exists(path):
        ProtoBuf(bytes.fromhex(path)).dump()
    elif os.path.isfile(path):
        with open(path, "rb") as file:
            content = file.read()
            print("file:", content.hex())
            ProtoBuf(content).dump()
    else:
        print("not a file:", path)



# https://github.com/synodriver/pyjce
from typing import List, Union

from .buffer import ByteBuffer
from .writer import JceWriter
from .reader import JceReader
from .struct import IJceStruct

__all__ = ["ByteBuffer", "IJceStruct", "JceReader", "JceWriter", "IAutoDecodeJceStruct"]

__version__ = "0.1.2"


class IAutoDecodeJceStruct(IJceStruct):
    """
    实现了一些标准方法的接口,当然要是序列化嵌套jce的话可能还是有问题
    """

    @classmethod
    def decode_list(cls, data: Union[bytes, bytearray], tag: int) -> List["IJceStruct"]:
        """
        从一个bytes里面解码出List[自己]
        :param data:
        :param tag:
        :return:
        """
        reader = JceReader(data)
        return reader.read_list(cls, tag)

    def read_from(self, reader: JceReader) -> None:
        for field_name, val in self.schema()["properties"].items():
            jce_id: int = val["jce_id"]
            setattr(self, field_name, reader.read_any(jce_id))

    def to_bytes(self) -> Union[bytes, bytearray]:
        writer = JceWriter()
        writer.write_jce_struct_raw(self)
        return writer.bytes()



from and_tools.byte_ import hex_format


class Pack:
    """组包
    好像作用不大....
    """

    def __init__(self):
        self._bytes_data = bytearray()

    def add_bin(self, bytes_temp):
        if bytes_temp is not None:
            self._bytes_data.extend(bytes_temp)

    def add_hex(self, bytes_temp):
        if bytes_temp is not None:
            self._bytes_data.extend(bytearray.fromhex(bytes_temp))

    def add_bytes(self, bytes_temp):
        """字节或字节集"""
        if bytes_temp is not None:
            self._bytes_data.append(bytes_temp)

    def add_int(self, int_temp, length=4):
        """整数
        length:
        int:4
        Short:2
        long:8
        """
        if int_temp is not None:
            self._bytes_data.extend(int_temp.to_bytes(length, 'big'))

    def add_body(self, data, length=4, _hex=False, add_len=0):
        """头部&内容"""
        if data is None:
            return

        if isinstance(data, str):
            bytes_data = bytes.fromhex(data) if _hex else data.encode('utf-8')
        else:
            bytes_data = data

        self.add_int(len(bytes_data) + add_len, length)
        self.add_bin(bytes_data)

    def set_data(self, byte_temp):
        """置数据"""
        if byte_temp is not None:
            self._bytes_data = byte_temp

    def empty(self):
        """清空"""
        self._bytes_data = bytearray()

    def get_bytes(self, _hex=False):
        if _hex:
            _bytes_temp = self._bytes_data.hex()
            _bytes_temp = hex_format(_bytes_temp)
        else:
            _bytes_temp = self._bytes_data
        return _bytes_temp



# package cooperation.qzone;
import time
import zlib
from typing import Union

from and_tools.Jce_b import JceWriter, JceReader, IJceStruct

from android.struct.QMF_PROTOCAL import QmfUpstream, QmfTokenInfo, QmfClientIpInfo, QmfBusiControl, RetryInfo, \
    QmfDownstream


def createBusiControl(compFlag, length, SUPPORT_COMPRESS):
    Qmf_Busi_Control = QmfBusiControl(
        compFlag=compFlag,
        lenBeforeComp=length,  # 补偿前的长度
        rspCompFlag=SUPPORT_COMPRESS
    ).to_bytes()
    _Buffer = JceWriter().write_jce_struct(Qmf_Busi_Control, 0)

    _Buffer = JceWriter().write_map({'busiCompCtl': {
        'QMF_PROTOCAL.QmfBusiControl': _Buffer
    }}, 0)
    return _Buffer


def createQmfUpstream(info, compFlag: int, BusiBuff: bytes, ServiceCmd: str):
    """"
            创建 Qmf
                compFlag: 比较标志  1 =压缩
                BusiBuff: 业务数据
    """

    Extra = createBusiControl(
        compFlag=compFlag,
        length=len(BusiBuff),  # 压缩前的长度
        SUPPORT_COMPRESS=1

    )

    TokenInfo = QmfTokenInfo(
        Type=64,
        Key=b'',
        ext_key={1: bytearray.fromhex('00')}
    )

    ClientIpInfo = QmfClientIpInfo(
        IpType=0,
        ClientPort=0,
        ClientIpv4=0,
        ClientIpv6=bytearray.fromhex('00 00 00 00 00 00')
    )

    _RetryInfo = RetryInfo(
        Flag=1,
        RetryCount=0,
        PkgId=int(time.time() * 1000)

    )
    if compFlag:
        BusiBuff = zlib.compress(BusiBuff)
    Upstream = QmfUpstream(
        Seq=info.seq,
        Appid=1000027,
        Uin=int(info.uin),
        Qua='V1_AND_SQ_8.9.83_4680_YYB_D',
        ServiceCmd=ServiceCmd,
        DeviceInfo='',
        Token=TokenInfo,
        IpInfo=ClientIpInfo,
        BusiBuff=BusiBuff,
        Extra=Extra,
        flag=0,
        sessionID=0,
        retryinfo=_RetryInfo,

    ).to_bytes()

    return Upstream


def UnQmfDownstream(Buffer: bytes, key_1: str, key_2: str):
    """解QMF"""

    stream = QmfDownstream()
    stream.read_from(JceReader(Buffer))

    Buffer_head = stream.BusiBuff[:1]
    if Buffer_head == bytearray(b'x'):
        BusiBuff = zlib.decompress(stream.BusiBuff)
    else:
        BusiBuff = stream.BusiBuff

    BusiBuff = JceReader(BusiBuff).read_map(0)

    rsp_Buffer = BusiBuff.get(key_1, {}).get(key_2, b'')
    msg_Buffer = BusiBuff.get('msg', {}).get('string', b'')
    ret_Buffer = BusiBuff.get('ret', {}).get('int32', b'')
    msg = JceReader(msg_Buffer).read_string(0)
    ret = JceReader(ret_Buffer).read_int64(0)

    return rsp_Buffer, msg, ret
